# -*- coding: utf-8 -*-

# PLEASE DO NOT EDIT THIS FILE, IT IS GENERATED AND WILL BE OVERWRITTEN:
# https://github.com/ccxt/ccxt/blob/master/CONTRIBUTING.md#how-to-contribute-code

from ccxt.base.exchange import Exchange
from ccxt.abstract.ace import ImplicitAPI
from ccxt.base.types import Balances, Int, Market, Num, Order, OrderBook, OrderSide, OrderType, Str, Strings, Ticker, Tickers, Trade
from typing import List
from ccxt.base.errors import AuthenticationError
from ccxt.base.errors import ArgumentsRequired
from ccxt.base.errors import BadRequest
from ccxt.base.errors import InsufficientFunds
from ccxt.base.errors import InvalidOrder
from ccxt.base.decimal_to_precision import TICK_SIZE
from ccxt.base.precise import Precise


class ace(Exchange, ImplicitAPI):

    def describe(self):
        return self.deep_extend(super(ace, self).describe(), {
            'id': 'ace',
            'name': 'ACE',
            'countries': ['TW'],  # Taiwan
            'version': 'v2',
            'rateLimit': 100,
            'pro': False,
            'has': {
                'CORS': None,
                'spot': True,
                'margin': False,
                'swap': False,
                'future': False,
                'option': False,
                'cancelAllOrders': False,
                'cancelOrder': True,
                'cancelOrders': False,
                'closeAllPositions': False,
                'closePosition': False,
                'createOrder': True,
                'editOrder': False,
                'fetchBalance': True,
                'fetchBorrowRateHistories': False,
                'fetchBorrowRateHistory': False,
                'fetchClosedOrders': False,
                'fetchCrossBorrowRate': False,
                'fetchCrossBorrowRates': False,
                'fetchCurrencies': False,
                'fetchDepositAddress': False,
                'fetchDeposits': False,
                'fetchFundingHistory': False,
                'fetchFundingRate': False,
                'fetchFundingRateHistory': False,
                'fetchFundingRates': False,
                'fetchIndexOHLCV': False,
                'fetchIsolatedBorrowRate': False,
                'fetchIsolatedBorrowRates': False,
                'fetchMarginMode': False,
                'fetchMarkets': True,
                'fetchMarkOHLCV': False,
                'fetchMyTrades': True,
                'fetchOHLCV': True,
                'fetchOpenInterestHistory': False,
                'fetchOpenOrders': True,
                'fetchOrder': True,
                'fetchOrderBook': True,
                'fetchOrders': False,
                'fetchOrderTrades': True,
                'fetchPosition': False,
                'fetchPositionHistory': False,
                'fetchPositionMode': False,
                'fetchPositions': False,
                'fetchPositionsForSymbol': False,
                'fetchPositionsHistory': False,
                'fetchPositionsRisk': False,
                'fetchPremiumIndexOHLCV': False,
                'fetchTicker': True,
                'fetchTickers': True,
                'fetchTime': False,
                'fetchTrades': False,
                'fetchTradingFee': False,
                'fetchTradingFees': False,
                'fetchTransactionFees': False,
                'fetchTransactions': False,
                'fetchTransfer': False,
                'fetchTransfers': False,
                'fetchWithdrawal': False,
                'fetchWithdrawals': False,
                'setLeverage': False,
                'setMarginMode': False,
                'transfer': False,
                'withdraw': False,
                'ws': False,
            },
            'timeframes': {
                '1m': 1,
                '5m': 5,
                '10m': 10,
                '30m': 10,
                '1h': 60,
                '2h': 120,
                '4h': 240,
                '8h': 480,
                '12h': 720,
                '1d': 24,
                '1w': 70,
                '1M': 31,
            },
            'urls': {
                'logo': 'https://user-images.githubusercontent.com/1294454/216908003-fb314cf6-e66e-471c-b91d-1d86e4baaa90.jpg',
                'api': {
                    'public': 'https://ace.io/polarisex',
                    'private': 'https://ace.io/polarisex/open',
                },
                'www': 'https://ace.io/',
                'doc': [
                    'https://github.com/ace-exchange/ace-offical-api-docs',
                ],
                'fees': 'https://helpcenter.ace.io/hc/zh-tw/articles/360018609132-%E8%B2%BB%E7%8E%87%E8%AA%AA%E6%98%8E',
            },
            'requiredCredentials': {
                'apiKey': True,
                'secret': True,
            },
            'api': {
                'public': {
                    'get': [
                        'oapi/v2/list/tradePrice',
                        'oapi/v2/list/marketPair',
                        'open/v2/public/getOrderBook',
                    ],
                },
                'private': {
                    'post': [
                        'v2/coin/customerAccount',
                        'v2/kline/getKline',
                        'v2/order/order',
                        'v2/order/cancel',
                        'v2/order/getOrderList',
                        'v2/order/showOrderStatus',
                        'v2/order/showOrderHistory',
                        'v2/order/getTradeList',
                    ],
                },
            },
            'fees': {
                'trading': {
                    'percentage': True,
                    'maker': self.parse_number('0.0005'),
                    'taker': self.parse_number('0.001'),
                },
            },
            'options': {
                'brokerId': 'ccxt',
            },
            'precisionMode': TICK_SIZE,
            'exceptions': {
                'exact': {
                    '2003': InvalidOrder,
                    '2004': InvalidOrder,
                    '2005': InvalidOrder,
                    '2021': InsufficientFunds,
                    '2036': InvalidOrder,
                    '2039': InvalidOrder,
                    '2053': InvalidOrder,
                    '2061': BadRequest,
                    '2063': InvalidOrder,
                    '9996': BadRequest,
                    '10012': AuthenticationError,
                    '20182': AuthenticationError,
                    '20183': InvalidOrder,
                },
                'broad': {
                },
            },
            'commonCurrencies': {
            },
        })

    def fetch_markets(self, params={}) -> List[Market]:
        """
        retrieves data on all markets for ace
        :see: https://github.com/ace-exchange/ace-official-api-docs/blob/master/api_v2.md#oapi-api---market-pair
        :param dict [params]: extra parameters specific to the exchange API endpoint
        :returns dict[]: an array of objects representing market data
        """
        response = self.publicGetOapiV2ListMarketPair()
        #
        #     [
        #         {
        #             "symbol":"BTC/USDT",
        #             "base":"btc",
        #             "baseCurrencyId": "122"
        #             "quote":"usdt",
        #             "basePrecision":"8",
        #             "quotePrecision":"5",
        #             "minLimitBaseAmount":"0.1",
        #             "maxLimitBaseAmount":"480286"
        #         }
        #     ]
        #
        return self.parse_markets(response)

    def parse_market(self, market) -> Market:
        baseId = self.safe_string(market, 'base')
        base = self.safe_currency_code(baseId)
        quoteId = self.safe_string(market, 'quote')
        quote = self.safe_currency_code(quoteId)
        symbol = base + '/' + quote
        return {
            'id': self.safe_string(market, 'symbol'),
            'uppercaseId': None,
            'symbol': symbol,
            'base': base,
            'baseId': baseId,
            'quote': quote,
            'quoteId': quoteId,
            'settle': None,
            'settleId': None,
            'type': 'spot',
            'spot': True,
            'margin': False,
            'swap': False,
            'future': False,
            'option': False,
            'contract': False,
            'linear': None,
            'inverse': None,
            'contractSize': None,
            'expiry': None,
            'expiryDatetime': None,
            'strike': None,
            'optionType': None,
            'limits': {
                'amount': {
                    'min': self.safe_number(market, 'minLimitBaseAmount'),
                    'max': self.safe_number(market, 'maxLimitBaseAmount'),
                },
                'price': {
                    'min': None,
                    'max': None,
                },
                'cost': {
                    'min': None,
                    'max': None,
                },
                'leverage': {
                    'min': None,
                    'max': None,
                },
            },
            'precision': {
                'price': self.parse_number(self.parse_precision(self.safe_string(market, 'quotePrecision'))),
                'amount': self.parse_number(self.parse_precision(self.safe_string(market, 'basePrecision'))),
            },
            'active': None,
            'created': None,
            'info': market,
        }

    def parse_ticker(self, ticker: dict, market: Market = None) -> Ticker:
        #
        #     {
        #         "base_volume":229196.34035399999,
        #         "last_price":11881.06,
        #         "quote_volume":19.2909
        #     }
        #
        marketId = self.safe_string(ticker, 'id')
        symbol = self.safe_symbol(marketId, market)
        return self.safe_ticker({
            'symbol': symbol,
            'timestamp': None,
            'datetime': None,
            'high': None,
            'low': None,
            'bid': None,
            'bidVolume': None,
            'ask': None,
            'askVolume': None,
            'vwap': None,
            'open': None,
            'close': self.safe_string(ticker, 'last_price'),
            'last': self.safe_string(ticker, 'last_price'),
            'previousClose': None,
            'change': None,
            'percentage': None,
            'average': None,
            'baseVolume': self.safe_string(ticker, 'base_volume'),
            'quoteVolume': self.safe_string(ticker, 'quote_volume'),
            'info': ticker,
        }, market)

    def fetch_ticker(self, symbol: str, params={}) -> Ticker:
        """
        fetches a price ticker, a statistical calculation with the information calculated over the past 24 hours for a specific market
        :see: https://github.com/ace-exchange/ace-official-api-docs/blob/master/api_v2.md#oapi-api---trade-data
        :param str symbol: unified symbol of the market to fetch the ticker for
        :param dict [params]: extra parameters specific to the exchange API endpoint
        :returns dict: a `ticker structure <https://docs.ccxt.com/#/?id=ticker-structure>`
        """
        self.load_markets()
        market = self.market(symbol)
        response = self.publicGetOapiV2ListTradePrice(params)
        marketId = market['id']
        ticker = self.safe_value(response, marketId, {})
        #
        #     {
        #         "BTC/USDT":{
        #             "base_volume":229196.34035399999,
        #             "last_price":11881.06,
        #             "quote_volume":19.2909
        #         }
        #     }
        #
        return self.parse_ticker(ticker, market)

    def fetch_tickers(self, symbols: Strings = None, params={}) -> Tickers:
        """
        fetches price tickers for multiple markets, statistical information calculated over the past 24 hours for each market
        :see: https://github.com/ace-exchange/ace-official-api-docs/blob/master/api_v2.md#oapi-api---trade-data
        :param str[]|None symbols: unified symbols of the markets to fetch the ticker for, all market tickers are returned if not assigned
        :param dict [params]: extra parameters specific to the exchange API endpoint
        :returns dict: a dictionary of `ticker structures <https://docs.ccxt.com/#/?id=ticker-structure>`
        """
        self.load_markets()
        response = self.publicGetOapiV2ListTradePrice()
        #
        #     {
        #         "BTC/USDT":{
        #             "base_volume":229196.34035399999,
        #             "last_price":11881.06,
        #             "quote_volume":19.2909
        #         }
        #     }
        #
        tickers = []
        pairs = list(response.keys())
        for i in range(0, len(pairs)):
            marketId = pairs[i]
            market = self.safe_market(marketId)
            rawTicker = self.safe_value(response, marketId)
            ticker = self.parse_ticker(rawTicker, market)
            tickers.append(ticker)
        return self.filter_by_array_tickers(tickers, 'symbol', symbols)

    def fetch_order_book(self, symbol: str, limit: Int = None, params={}) -> OrderBook:
        """
        fetches information on open orders with bid(buy) and ask(sell) prices, volumes and other data
        :see: https://github.com/ace-exchange/ace-official-api-docs/blob/master/api_v2.md#open-api---order-books
        :param str symbol: unified symbol of the market to fetch the order book for
        :param int [limit]: the maximum amount of order book entries to return
        :param dict [params]: extra parameters specific to the exchange API endpoint
        :returns dict: A dictionary of `order book structures <https://docs.ccxt.com/#/?id=order-book-structure>` indexed by market symbols
        """
        self.load_markets()
        market = self.market(symbol)
        request = {
            'quoteCurrencyId': market['quoteId'],
            'baseCurrencyId': market['baseId'],
        }
        if limit is not None:
            request['depth'] = limit
        response = self.publicGetOpenV2PublicGetOrderBook(self.extend(request, params))
        #
        #     {
        #         "attachment": {
        #             "baseCurrencyId": "2",
        #             "quoteCurrencyId": "14",
        #             "baseCurrencyName": "BTC",
        #             "quoteCurrencyName": "USDT",
        #             "bids": [
        #                 [
        #                     "0.0009",
        #                     "19993.53"
        #                 ],
        #                 [
        #                     "0.001",
        #                     "19675.33"
        #                 ],
        #                 [
        #                     "0.001",
        #                     "19357.13"
        #                 ]
        #             ],
        #             "asks": [
        #                 [
        #                     "0.001",
        #                     "20629.92"
        #                 ],
        #                 [
        #                     "0.001",
        #                     "20948.12"
        #                 ]
        #             ]
        #         },
        #         "message": null,
        #         "parameters": null,
        #         "status": 200
        #     }
        #
        orderBook = self.safe_dict(response, 'attachment')
        return self.parse_order_book(orderBook, market['symbol'], None, 'bids', 'asks')

    def parse_ohlcv(self, ohlcv, market: Market = None) -> list:
        #
        #     {
        #         "changeRate": 0,
        #         "volume": 0,
        #         "closePrice": 101000.0,
        #         "lowPrice": 101000.0,
        #         "highPrice": 101000.0,
        #         "highPrice": 1573195740000L,
        #         "openPrice": 101000.0,
        #         "current": 101000.0,
        #         "currentTime": "2019-11-08 14:49:00",
        #         "createTime": "2019-11-08 14:49:00"
        #     }
        #
        dateTime = self.safe_string(ohlcv, 'createTime')
        timestamp = self.parse8601(dateTime)
        if timestamp is not None:
            timestamp = timestamp - 28800000  # 8 hours
        return [
            timestamp,
            self.safe_number(ohlcv, 'openPrice'),
            self.safe_number(ohlcv, 'highPrice'),
            self.safe_number(ohlcv, 'lowPrice'),
            self.safe_number(ohlcv, 'closePrice'),
            self.safe_number(ohlcv, 'volume'),
        ]

    def fetch_ohlcv(self, symbol: str, timeframe='1m', since: Int = None, limit: Int = None, params={}) -> List[list]:
        """
        fetches historical candlestick data containing the open, high, low, and close price, and the volume of a market
        :see: https://github.com/ace-exchange/ace-official-api-docs/blob/master/api_v2.md#open-api---klinecandlestick-data
        :param str symbol: unified symbol of the market to fetch OHLCV data for
        :param str timeframe: the length of time each candle represents
        :param int [since]: timestamp in ms of the earliest candle to fetch
        :param int [limit]: the maximum amount of candles to fetch
        :param dict [params]: extra parameters specific to the exchange API endpoint
        :returns int[][]: A list of candles ordered, open, high, low, close, volume
        """
        self.load_markets()
        market = self.market(symbol)
        request = {
            'duration': self.timeframes[timeframe],
            'quoteCurrencyId': market['quoteId'],
            'baseCurrencyId': market['baseId'],
        }
        if limit is not None:
            request['limit'] = limit
        if since is not None:
            request['startTime'] = since
        response = self.privatePostV2KlineGetKline(self.extend(request, params))
        data = self.safe_value(response, 'attachment', [])
        #
        #     {
        #         "attachment":[
        #                 {
        #                     "changeRate": 0,
        #                     "closePrice": 101000.0,
        #                     "volume": 0,
        #                     "lowPrice": 101000.0,
        #                     "highPrice": 101000.0,
        #                     "highPrice": 1573195740000L,
        #                     "openPrice": 101000.0,
        #                     "current": 101000.0,
        #                     "currentTime": "2019-11-08 14:49:00",
        #                     "createTime": "2019-11-08 14:49:00"
        #                 }
        #         ]
        #     }
        #
        return self.parse_ohlcvs(data, market, timeframe, since, limit)

    def parse_order_status(self, status):
        statuses = {
            '0': 'open',
            '1': 'open',
            '2': 'closed',
            '4': 'canceled',
            '5': 'canceled',
        }
        return self.safe_string(statuses, status, None)

    def parse_order(self, order, market: Market = None) -> Order:
        #
        # createOrder
        #         "15697850529570392100421100482693"
        #
        # fetchOpenOrders
        #         {
        #             "uid": 0,
        #             "orderNo": "16113081376560890227301101413941",
        #             "orderTime": "2021-01-22 17:35:37",
        #             "orderTimeStamp": 1611308137656,
        #             "baseCurrencyId": 1,
        #             "baseCurrencyName": "TWD",
        #             "quoteCurrencyId": 14,
        #             "quoteCurrencyName": "USDT",
        #             "buyOrSell": "1",
        #             "num": "6.0000000000000000",
        #             "price": "32.5880000000000000",
        #             "remainNum": "2.0000000000000000",
        #             "tradeNum": "4.0000000000000000",
        #             "tradePrice": "31.19800000000000000000",
        #             "tradeAmount": "124.7920000000000000",
        #             "tradeRate": "0.66666666666666666667",
        #             "status": 1,
        #             "type": 1
        #         }
        #
        id: Str
        timestamp: Int = None
        symbol: Str = None
        price: Str = None
        amount: Str = None
        side: Str = None
        type: Str = None
        status: Str = None
        filled: Str = None
        remaining: Str = None
        average: Str = None
        if isinstance(order, str):
            id = order
        else:
            id = self.safe_string(order, 'orderNo')
            timestamp = self.safe_integer(order, 'orderTimeStamp')
            if timestamp is None:
                dateTime = self.safe_string(order, 'orderTime')
                if dateTime is not None:
                    timestamp = self.parse8601(dateTime)
                    timestamp = timestamp - 28800000  # 8 hours
            orderSide = self.safe_number(order, 'buyOrSell')
            if orderSide is not None:
                side = 'buy' if (orderSide == 1) else 'sell'
            amount = self.safe_string(order, 'num')
            price = self.safe_string(order, 'price')
            quoteId = self.safe_string(order, 'quoteCurrencyName')
            baseId = self.safe_string(order, 'baseCurrencyName')
            if quoteId is not None and baseId is not None:
                symbol = baseId + '/' + quoteId
            orderType = self.safe_number(order, 'type')
            if orderType is not None:
                type = 'limit' if (orderType == 1) else 'market'
            filled = self.safe_string(order, 'tradeNum')
            remaining = self.safe_string(order, 'remainNum')
            status = self.parse_order_status(self.safe_string(order, 'status'))
            average = self.safe_string(order, 'averagePrice')
        return self.safe_order({
            'id': id,
            'clientOrderId': None,
            'timestamp': timestamp,
            'datetime': self.iso8601(timestamp),
            'lastTradeTimestamp': None,
            'symbol': symbol,
            'type': type,
            'timeInForce': None,
            'postOnly': None,
            'side': side,
            'price': price,
            'stopPrice': None,
            'amount': amount,
            'cost': None,
            'average': average,
            'filled': filled,
            'remaining': remaining,
            'status': status,
            'fee': None,
            'trades': None,
            'info': order,
        }, market)

    def create_order(self, symbol: str, type: OrderType, side: OrderSide, amount: float, price: Num = None, params={}):
        """
        create a trade order
        :see: https://github.com/ace-exchange/ace-official-api-docs/blob/master/api_v2.md#open-api---new-order
        :param str symbol: unified symbol of the market to create an order in
        :param str type: 'market' or 'limit'
        :param str side: 'buy' or 'sell'
        :param float amount: how much of currency you want to trade in units of base currency
        :param float [price]: the price at which the order is to be fullfilled, in units of the quote currency, ignored in market orders
        :param dict [params]: extra parameters specific to the exchange API endpoint
        :returns dict: an `order structure <https://docs.ccxt.com/#/?id=order-structure>`
        """
        self.load_markets()
        market = self.market(symbol)
        orderType = type.upper()
        orderSide = side.upper()
        request = {
            'baseCurrencyId': market['baseId'],
            'quoteCurrencyId': market['quoteId'],
            'type': 1 if (orderType == 'LIMIT') else 2,
            'buyOrSell': 1 if (orderSide == 'BUY') else 2,
            'num': self.amount_to_precision(symbol, amount),
        }
        if type == 'limit':
            request['price'] = self.price_to_precision(symbol, price)
        response = self.privatePostV2OrderOrder(self.extend(request, params))
        #
        #     {
        #         "attachment": "15697850529570392100421100482693",
        #         "message": null,
        #         "parameters": null,
        #         "status": 200
        #     }
        #
        data = self.safe_dict(response, 'attachment')
        return self.parse_order(data, market)

    def cancel_order(self, id: str, symbol: Str = None, params={}):
        """
        cancels an open order
        :see: https://github.com/ace-exchange/ace-official-api-docs/blob/master/api_v2.md#open-api---cancel-order
        :param str id: order id
        :param str symbol: unified symbol of the market the order was made in
        :param dict [params]: extra parameters specific to the exchange API endpoint
        :returns dict: An `order structure <https://docs.ccxt.com/#/?id=order-structure>`
        """
        self.load_markets()
        request = {
            'orderNo': id,
        }
        response = self.privatePostV2OrderCancel(self.extend(request, params))
        #
        #     {
        #         "attachment": 200,
        #         "message": null,
        #         "parameters": null,
        #         "status": 200
        #     }
        #
        return response

    def fetch_order(self, id: str, symbol: Str = None, params={}):
        """
        fetches information on an order made by the user
        :see: https://github.com/ace-exchange/ace-official-api-docs/blob/master/api_v2.md#open-api---order-status
        :param str symbol: unified symbol of the market the order was made in
        :param dict [params]: extra parameters specific to the exchange API endpoint
        :returns dict: An `order structure <https://docs.ccxt.com/#/?id=order-structure>`
        """
        self.load_markets()
        request = {
            'orderNo': id,
        }
        response = self.privatePostV2OrderShowOrderStatus(self.extend(request, params))
        #
        #     {
        #         "attachment": {
        #             "buyOrSell": 1,
        #             "averagePrice": "490849.75000000",
        #             "num": "0.00000000",
        #             "orderTime": "2022-11-29 18:03:06.318",
        #             "price": "490849.75000000",
        #             "status": 4,
        #             "tradeNum": "0.02697000",
        #             "remainNum": "0.97303000",
        #             "baseCurrencyId": 2,
        #             "baseCurrencyName": "BTC",
        #             "quoteCurrencyId": 1,
        #             "quoteCurrencyName": "TWD",
        #             "orderNo": "16697161898600391472461100244406"
        #         },
        #         "message": null,
        #         "parameters": null,
        #         "status": 200
        #     }
        #
        data = self.safe_dict(response, 'attachment')
        return self.parse_order(data, None)

    def fetch_open_orders(self, symbol: Str = None, since: Int = None, limit: Int = None, params={}) -> List[Order]:
        """
        fetch all unfilled currently open orders
        :see: https://github.com/ace-exchange/ace-official-api-docs/blob/master/api_v2.md#open-api---order-list
        :param str symbol: unified market symbol of the market orders were made in
        :param int [since]: the earliest time in ms to fetch orders for
        :param int [limit]: the maximum number of order structures to retrieve
        :param dict [params]: extra parameters specific to the exchange API endpoint
        :returns Order[]: a list of `order structures <https://docs.ccxt.com/#/?id=order-structure>`
        """
        if symbol is None:
            raise ArgumentsRequired(self.id + ' fetchOpenOrders() requires a symbol argument')
        self.load_markets()
        market = self.market(symbol)
        request = {
            'quoteCurrencyId': market['quoteId'],
            'baseCurrencyId': market['baseId'],
            # 'start': 0,
        }
        if limit is not None:
            request['size'] = limit
        response = self.privatePostV2OrderGetOrderList(self.extend(request, params))
        orders = self.safe_value(response, 'attachment')
        #
        #     {
        #         "attachment": [
        #             {
        #                 "uid": 0,
        #                 "orderNo": "16113081376560890227301101413941",
        #                 "orderTime": "2021-01-22 17:35:37",
        #                 "orderTimeStamp": 1611308137656,
        #                 "baseCurrencyId": 1,
        #                 "baseCurrencyName": "TWD",
        #                 "quoteCurrencyId": 14,
        #                 "quoteCurrencyName": "USDT",
        #                 "buyOrSell": "1",
        #                 "num": "6.0000000000000000",
        #                 "price": "32.5880000000000000",
        #                 "remainNum": "2.0000000000000000",
        #                 "tradeNum": "4.0000000000000000",
        #                 "tradePrice": "31.19800000000000000000",
        #                 "tradeAmount": "124.7920000000000000",
        #                 "tradeRate": "0.66666666666666666667",
        #                 "status": 1,
        #                 "type": 1
        #             }
        #         ],
        #         "message": null,
        #         "parameters": null,
        #         "status": 200
        #     }
        #
        return self.parse_orders(orders, market, since, limit)

    def parse_trade(self, trade, market: Market = None) -> Trade:
        #
        # fetchOrderTrades
        #         {
        #             "amount": 0.0030965,
        #             "tradeNo": "15681920522485652100751000417788",
        #             "price": "0.03096500",
        #             "num": "0.10000000",
        #             "bi": 1,
        #             "time": "2019-09-11 16:54:12.248"
        #         }
        #
        # fetchMyTrades
        #         {
        #             "buyOrSell": 1,
        #             "orderNo": "16708156853695560053601100247906",
        #             "num": "1",
        #             "price": "16895",
        #             "orderAmount": "16895",
        #             "tradeNum": "0.1",
        #             "tradePrice": "16895",
        #             "tradeAmount": "1689.5",
        #             "fee": "0",
        #             "feeSave": "0",
        #             "status": 1,
        #             "isSelf": False,
        #             "tradeNo": "16708186395087940051961000274150",
        #             "tradeTime": "2022-12-12 12:17:19",
        #             "tradeTimestamp": 1670818639508,
        #             "quoteCurrencyId": 14,
        #             "quoteCurrencyName": "USDT",
        #             "baseCurrencyId": 2,
        #             "baseCurrencyName": "BTC"
        #         }
        id = self.safe_string(trade, 'tradeNo')
        price = self.safe_string(trade, 'price')
        amount = self.safe_string(trade, 'num')
        timestamp = self.safe_integer(trade, 'tradeTimestamp')
        if timestamp is None:
            datetime = self.safe_string_2(trade, 'time', 'tradeTime')
            timestamp = self.parse8601(datetime)
            timestamp = timestamp - 28800000  # 8 hours normalize timestamp
        symbol = market['symbol']
        quoteId = self.safe_string(trade, 'quoteCurrencyName')
        baseId = self.safe_string(trade, 'baseCurrencyName')
        if quoteId is not None and baseId is not None:
            symbol = baseId + '/' + quoteId
        side: Str = None
        tradeSide = self.safe_integer(trade, 'buyOrSell')
        if tradeSide is not None:
            side = 'buy' if (tradeSide == 1) else 'sell'
        feeString = self.safe_string(trade, 'fee')
        fee = None
        if feeString is not None:
            feeSaveString = self.safe_string(trade, 'feeSave')
            fee = {
                'cost': Precise.string_sub(feeString, feeSaveString),
                'currency': quoteId,
            }
        return self.safe_trade({
            'info': trade,
            'id': id,
            'order': self.safe_string(trade, 'orderNo'),
            'symbol': symbol,
            'side': side,
            'type': None,
            'takerOrMaker': None,
            'price': price,
            'amount': amount,
            'cost': None,
            'fee': fee,
            'timestamp': timestamp,
            'datetime': self.iso8601(timestamp),
        }, market)

    def fetch_order_trades(self, id: str, symbol: Str = None, since: Int = None, limit: Int = None, params={}):
        """
        fetch all the trades made from a single order
        :see: https://github.com/ace-exchange/ace-official-api-docs/blob/master/api_v2.md#open-api---order-history
        :param str id: order id
        :param str symbol: unified market symbol
        :param int [since]: the earliest time in ms to fetch trades for
        :param int [limit]: the maximum number of trades to retrieve
        :param dict [params]: extra parameters specific to the exchange API endpoint
        :returns dict[]: a list of `trade structures <https://docs.ccxt.com/#/?id=trade-structure>`
        """
        self.load_markets()
        market = self.safe_market(symbol)
        request = {
            'orderNo': id,
        }
        response = self.privatePostV2OrderShowOrderHistory(self.extend(request, params))
        #
        #     {
        #         "attachment": {
        #             "order": {
        #                 "buyOrSell": 1,
        #                 "averagePrice": "491343.74000000",
        #                 "num": "1.00000000",
        #                 "orderTime": "2022-11-29 18:32:22.232",
        #                 "price": "491343.74000000",
        #                 "status": 1,
        #                 "tradeNum": "0.01622200",
        #                 "remainNum": "0.98377800",
        #                 "baseCurrencyId": 2,
        #                 "baseCurrencyName": "BTC",
        #                 "quoteCurrencyId": 1,
        #                 "quoteCurrencyName": "TWD",
        #                 "orderNo": "16697179457740441472471100214402"
        #             },
        #             "trades": [
        #                 {
        #                     "price": "491343.74000000",
        #                     "num": "0.01622200",
        #                     "time": "2022-11-29 18:32:25.789",
        #                     "tradeNo": "16697179457897791471461000223437",
        #                     "amount": "7970.57815028"
        #                 }
        #             ]
        #         },
        #         "message": null,
        #         "parameters": null,
        #         "status": 200
        #     }
        #
        data = self.safe_value(response, 'attachment')
        trades = self.safe_list(data, 'trades', [])
        return self.parse_trades(trades, market, since, limit)

    def fetch_my_trades(self, symbol: Str = None, since: Int = None, limit: Int = None, params={}):
        """
        fetch all trades made by the user
        :see: https://github.com/ace-exchange/ace-official-api-docs/blob/master/api_v2.md#open-api---trade-list
        :param str symbol: unified symbol of the market to fetch trades for
        :param int [since]: timestamp in ms of the earliest trade to fetch
        :param int [limit]: the maximum amount of trades to fetch
        :param dict [params]: extra parameters specific to the exchange API endpoint
        :returns Trade[]: a list of `trade structures <https://docs.ccxt.com/#/?id=public-trades>`
        """
        self.load_markets()
        market = self.safe_market(symbol)
        request = {
            # 'buyOrSell': 1,
            # 'start': 0,
        }
        if market['id'] is not None:
            request['quoteCurrencyId'] = market['quoteId']
            request['baseCurrencyId'] = market['baseId']
        if limit is not None:
            request['size'] = limit  # default 10, max 500
        response = self.privatePostV2OrderGetTradeList(self.extend(request, params))
        #
        #     {
        #         "attachment": [
        #             {
        #                 "buyOrSell": 1,
        #                 "orderNo": "16708156853695560053601100247906",
        #                 "num": "1",
        #                 "price": "16895",
        #                 "orderAmount": "16895",
        #                 "tradeNum": "0.1",
        #                 "tradePrice": "16895",
        #                 "tradeAmount": "1689.5",
        #                 "fee": "0",
        #                 "feeSave": "0",
        #                 "status": 1,
        #                 "isSelf": False,
        #                 "tradeNo": "16708186395087940051961000274150",
        #                 "tradeTime": "2022-12-12 12:17:19",
        #                 "tradeTimestamp": 1670818639508,
        #                 "quoteCurrencyId": 14,
        #                 "quoteCurrencyName": "USDT",
        #                 "baseCurrencyId": 2,
        #                 "baseCurrencyName": "BTC"
        #             }
        #         ],
        #         "message": null,
        #         "parameters": null,
        #         "status": 200
        #     }
        #
        trades = self.safe_list(response, 'attachment', [])
        return self.parse_trades(trades, market, since, limit)

    def parse_balance(self, response) -> Balances:
        #
        #     [
        #         {
        #             "currencyId": 4,
        #             "amount": 6.896,
        #             "cashAmount": 6.3855,
        #             "uid": 123,
        #             "currencyName": "BTC"
        #         }
        #     ]
        #
        result = {
            'info': response,
        }
        for i in range(0, len(response)):
            balance = response[i]
            currencyId = self.safe_string(balance, 'currencyName')
            code = self.safe_currency_code(currencyId)
            amount = self.safe_string(balance, 'amount')
            available = self.safe_string(balance, 'cashAmount')
            account = {
                'free': available,
                'total': amount,
            }
            result[code] = account
        return self.safe_balance(result)

    def fetch_balance(self, params={}) -> Balances:
        """
        query for balance and get the amount of funds available for trading or funds locked in orders
        :see: https://github.com/ace-exchange/ace-official-api-docs/blob/master/api_v2.md#open-api---account-balance
        :param dict [params]: extra parameters specific to the exchange API endpoint
        :returns dict: a `balance structure <https://docs.ccxt.com/#/?id=balance-structure>`
        """
        self.load_markets()
        response = self.privatePostV2CoinCustomerAccount(params)
        balances = self.safe_value(response, 'attachment', [])
        #
        #     {
        #         "attachment":[
        #             {
        #                 "currencyId": 4,
        #                 "amount": 6.896,
        #                 "cashAmount": 6.3855,
        #                 "uid": 123,
        #                 "currencyName": "BTC"
        #             }
        #         ],
        #         "message": null,
        #         "parameters": null,
        #         "status": "200"
        #     }
        #
        return self.parse_balance(balances)

    def sign(self, path, api='public', method='GET', params={}, headers=None, body=None):
        url = '/' + self.implode_params(path, params)
        query = self.omit(params, self.extract_params(path))
        if headers is None:
            headers = {}
        if api == 'private':
            self.check_required_credentials()
            nonce = self.milliseconds()
            auth = 'ACE_SIGN' + self.secret
            data = self.extend({
                'apiKey': self.apiKey,
                'timeStamp': nonce,
            }, params)
            dataKeys = list(data.keys())
            sortedDataKeys = self.sort_by(dataKeys, 0, False, '')
            for i in range(0, len(sortedDataKeys)):
                key = sortedDataKeys[i]
                auth += self.safe_string(data, key)
            signature = self.hash(self.encode(auth), 'sha256', 'hex')
            data['signKey'] = signature
            headers = {
                'Content-Type': 'application/x-www-form-urlencoded',
            }
            if method == 'POST':
                brokerId = self.safe_string(self.options, 'brokerId')
                if brokerId is not None:
                    headers['Referer'] = brokerId
            body = self.urlencode(data)
        elif api == 'public' and method == 'GET':
            if query:
                url += '?' + self.urlencode(query)
        url = self.urls['api'][api] + url
        return {'url': url, 'method': method, 'body': body, 'headers': headers}

    def handle_errors(self, code, reason, url, method, headers, body, response, requestHeaders, requestBody):
        if response is None:
            return None  # fallback to the default error handler
        feedback = self.id + ' ' + body
        status = self.safe_number(response, 'status', 200)
        if status > 200:
            statusStr = str(status)
            self.throw_exactly_matched_exception(self.exceptions['exact'], statusStr, feedback)
            self.throw_broadly_matched_exception(self.exceptions['broad'], statusStr, feedback)
        return None



from ccxt.base.types import Entry


class ImplicitAPI:
    public_get_spot_v3_public_symbols = publicGetSpotV3PublicSymbols = Entry('spot/v3/public/symbols', 'public', 'GET', {'cost': 1})
    public_get_spot_v3_public_quote_depth = publicGetSpotV3PublicQuoteDepth = Entry('spot/v3/public/quote/depth', 'public', 'GET', {'cost': 1})
    public_get_spot_v3_public_quote_depth_merged = publicGetSpotV3PublicQuoteDepthMerged = Entry('spot/v3/public/quote/depth/merged', 'public', 'GET', {'cost': 1})
    public_get_spot_v3_public_quote_trades = publicGetSpotV3PublicQuoteTrades = Entry('spot/v3/public/quote/trades', 'public', 'GET', {'cost': 1})
    public_get_spot_v3_public_quote_kline = publicGetSpotV3PublicQuoteKline = Entry('spot/v3/public/quote/kline', 'public', 'GET', {'cost': 1})
    public_get_spot_v3_public_quote_ticker_24hr = publicGetSpotV3PublicQuoteTicker24hr = Entry('spot/v3/public/quote/ticker/24hr', 'public', 'GET', {'cost': 1})
    public_get_spot_v3_public_quote_ticker_price = publicGetSpotV3PublicQuoteTickerPrice = Entry('spot/v3/public/quote/ticker/price', 'public', 'GET', {'cost': 1})
    public_get_spot_v3_public_quote_ticker_bookticker = publicGetSpotV3PublicQuoteTickerBookTicker = Entry('spot/v3/public/quote/ticker/bookTicker', 'public', 'GET', {'cost': 1})
    public_get_spot_v3_public_server_time = publicGetSpotV3PublicServerTime = Entry('spot/v3/public/server-time', 'public', 'GET', {'cost': 1})
    public_get_spot_v3_public_infos = publicGetSpotV3PublicInfos = Entry('spot/v3/public/infos', 'public', 'GET', {'cost': 1})
    public_get_spot_v3_public_margin_product_infos = publicGetSpotV3PublicMarginProductInfos = Entry('spot/v3/public/margin-product-infos', 'public', 'GET', {'cost': 1})
    public_get_spot_v3_public_margin_ensure_tokens = publicGetSpotV3PublicMarginEnsureTokens = Entry('spot/v3/public/margin-ensure-tokens', 'public', 'GET', {'cost': 1})
    public_get_v3_public_time = publicGetV3PublicTime = Entry('v3/public/time', 'public', 'GET', {'cost': 1})
    public_get_contract_v3_public_copytrading_symbol_list = publicGetContractV3PublicCopytradingSymbolList = Entry('contract/v3/public/copytrading/symbol/list', 'public', 'GET', {'cost': 1})
    public_get_derivatives_v3_public_order_book_l2 = publicGetDerivativesV3PublicOrderBookL2 = Entry('derivatives/v3/public/order-book/L2', 'public', 'GET', {'cost': 1})
    public_get_derivatives_v3_public_kline = publicGetDerivativesV3PublicKline = Entry('derivatives/v3/public/kline', 'public', 'GET', {'cost': 1})
    public_get_derivatives_v3_public_tickers = publicGetDerivativesV3PublicTickers = Entry('derivatives/v3/public/tickers', 'public', 'GET', {'cost': 1})
    public_get_derivatives_v3_public_instruments_info = publicGetDerivativesV3PublicInstrumentsInfo = Entry('derivatives/v3/public/instruments-info', 'public', 'GET', {'cost': 1})
    public_get_derivatives_v3_public_mark_price_kline = publicGetDerivativesV3PublicMarkPriceKline = Entry('derivatives/v3/public/mark-price-kline', 'public', 'GET', {'cost': 1})
    public_get_derivatives_v3_public_index_price_kline = publicGetDerivativesV3PublicIndexPriceKline = Entry('derivatives/v3/public/index-price-kline', 'public', 'GET', {'cost': 1})
    public_get_derivatives_v3_public_funding_history_funding_rate = publicGetDerivativesV3PublicFundingHistoryFundingRate = Entry('derivatives/v3/public/funding/history-funding-rate', 'public', 'GET', {'cost': 1})
    public_get_derivatives_v3_public_risk_limit_list = publicGetDerivativesV3PublicRiskLimitList = Entry('derivatives/v3/public/risk-limit/list', 'public', 'GET', {'cost': 1})
    public_get_derivatives_v3_public_delivery_price = publicGetDerivativesV3PublicDeliveryPrice = Entry('derivatives/v3/public/delivery-price', 'public', 'GET', {'cost': 1})
    public_get_derivatives_v3_public_recent_trade = publicGetDerivativesV3PublicRecentTrade = Entry('derivatives/v3/public/recent-trade', 'public', 'GET', {'cost': 1})
    public_get_derivatives_v3_public_open_interest = publicGetDerivativesV3PublicOpenInterest = Entry('derivatives/v3/public/open-interest', 'public', 'GET', {'cost': 1})
    public_get_derivatives_v3_public_insurance = publicGetDerivativesV3PublicInsurance = Entry('derivatives/v3/public/insurance', 'public', 'GET', {'cost': 1})
    public_get_v5_announcements_index = publicGetV5AnnouncementsIndex = Entry('v5/announcements/index', 'public', 'GET', {'cost': 5})
    public_get_v5_market_time = publicGetV5MarketTime = Entry('v5/market/time', 'public', 'GET', {'cost': 5})
    public_get_v5_market_kline = publicGetV5MarketKline = Entry('v5/market/kline', 'public', 'GET', {'cost': 5})
    public_get_v5_market_mark_price_kline = publicGetV5MarketMarkPriceKline = Entry('v5/market/mark-price-kline', 'public', 'GET', {'cost': 5})
    public_get_v5_market_index_price_kline = publicGetV5MarketIndexPriceKline = Entry('v5/market/index-price-kline', 'public', 'GET', {'cost': 5})
    public_get_v5_market_premium_index_price_kline = publicGetV5MarketPremiumIndexPriceKline = Entry('v5/market/premium-index-price-kline', 'public', 'GET', {'cost': 5})
    public_get_v5_market_instruments_info = publicGetV5MarketInstrumentsInfo = Entry('v5/market/instruments-info', 'public', 'GET', {'cost': 5})
    public_get_v5_market_orderbook = publicGetV5MarketOrderbook = Entry('v5/market/orderbook', 'public', 'GET', {'cost': 5})
    public_get_v5_market_tickers = publicGetV5MarketTickers = Entry('v5/market/tickers', 'public', 'GET', {'cost': 5})
    public_get_v5_market_funding_history = publicGetV5MarketFundingHistory = Entry('v5/market/funding/history', 'public', 'GET', {'cost': 5})
    public_get_v5_market_recent_trade = publicGetV5MarketRecentTrade = Entry('v5/market/recent-trade', 'public', 'GET', {'cost': 5})
    public_get_v5_market_open_interest = publicGetV5MarketOpenInterest = Entry('v5/market/open-interest', 'public', 'GET', {'cost': 5})
    public_get_v5_market_historical_volatility = publicGetV5MarketHistoricalVolatility = Entry('v5/market/historical-volatility', 'public', 'GET', {'cost': 5})
    public_get_v5_market_insurance = publicGetV5MarketInsurance = Entry('v5/market/insurance', 'public', 'GET', {'cost': 5})
    public_get_v5_market_risk_limit = publicGetV5MarketRiskLimit = Entry('v5/market/risk-limit', 'public', 'GET', {'cost': 5})
    public_get_v5_market_delivery_price = publicGetV5MarketDeliveryPrice = Entry('v5/market/delivery-price', 'public', 'GET', {'cost': 5})
    public_get_v5_market_account_ratio = publicGetV5MarketAccountRatio = Entry('v5/market/account-ratio', 'public', 'GET', {'cost': 5})
    public_get_v5_spot_lever_token_info = publicGetV5SpotLeverTokenInfo = Entry('v5/spot-lever-token/info', 'public', 'GET', {'cost': 5})
    public_get_v5_spot_lever_token_reference = publicGetV5SpotLeverTokenReference = Entry('v5/spot-lever-token/reference', 'public', 'GET', {'cost': 5})
    public_get_v5_spot_margin_trade_data = publicGetV5SpotMarginTradeData = Entry('v5/spot-margin-trade/data', 'public', 'GET', {'cost': 5})
    public_get_v5_spot_cross_margin_trade_data = publicGetV5SpotCrossMarginTradeData = Entry('v5/spot-cross-margin-trade/data', 'public', 'GET', {'cost': 5})
    public_get_v5_spot_cross_margin_trade_pledge_token = publicGetV5SpotCrossMarginTradePledgeToken = Entry('v5/spot-cross-margin-trade/pledge-token', 'public', 'GET', {'cost': 5})
    public_get_v5_spot_cross_margin_trade_borrow_token = publicGetV5SpotCrossMarginTradeBorrowToken = Entry('v5/spot-cross-margin-trade/borrow-token', 'public', 'GET', {'cost': 5})
    public_get_v5_ins_loan_product_infos = publicGetV5InsLoanProductInfos = Entry('v5/ins-loan/product-infos', 'public', 'GET', {'cost': 5})
    public_get_v5_ins_loan_ensure_tokens_convert = publicGetV5InsLoanEnsureTokensConvert = Entry('v5/ins-loan/ensure-tokens-convert', 'public', 'GET', {'cost': 5})
    private_get_v2_private_wallet_fund_records = privateGetV2PrivateWalletFundRecords = Entry('v2/private/wallet/fund/records', 'private', 'GET', {'cost': 25})
    private_get_spot_v3_private_order = privateGetSpotV3PrivateOrder = Entry('spot/v3/private/order', 'private', 'GET', {'cost': 2.5})
    private_get_spot_v3_private_open_orders = privateGetSpotV3PrivateOpenOrders = Entry('spot/v3/private/open-orders', 'private', 'GET', {'cost': 2.5})
    private_get_spot_v3_private_history_orders = privateGetSpotV3PrivateHistoryOrders = Entry('spot/v3/private/history-orders', 'private', 'GET', {'cost': 2.5})
    private_get_spot_v3_private_my_trades = privateGetSpotV3PrivateMyTrades = Entry('spot/v3/private/my-trades', 'private', 'GET', {'cost': 2.5})
    private_get_spot_v3_private_account = privateGetSpotV3PrivateAccount = Entry('spot/v3/private/account', 'private', 'GET', {'cost': 2.5})
    private_get_spot_v3_private_reference = privateGetSpotV3PrivateReference = Entry('spot/v3/private/reference', 'private', 'GET', {'cost': 2.5})
    private_get_spot_v3_private_record = privateGetSpotV3PrivateRecord = Entry('spot/v3/private/record', 'private', 'GET', {'cost': 2.5})
    private_get_spot_v3_private_cross_margin_orders = privateGetSpotV3PrivateCrossMarginOrders = Entry('spot/v3/private/cross-margin-orders', 'private', 'GET', {'cost': 10})
    private_get_spot_v3_private_cross_margin_account = privateGetSpotV3PrivateCrossMarginAccount = Entry('spot/v3/private/cross-margin-account', 'private', 'GET', {'cost': 10})
    private_get_spot_v3_private_cross_margin_loan_info = privateGetSpotV3PrivateCrossMarginLoanInfo = Entry('spot/v3/private/cross-margin-loan-info', 'private', 'GET', {'cost': 10})
    private_get_spot_v3_private_cross_margin_repay_history = privateGetSpotV3PrivateCrossMarginRepayHistory = Entry('spot/v3/private/cross-margin-repay-history', 'private', 'GET', {'cost': 10})
    private_get_spot_v3_private_margin_loan_infos = privateGetSpotV3PrivateMarginLoanInfos = Entry('spot/v3/private/margin-loan-infos', 'private', 'GET', {'cost': 10})
    private_get_spot_v3_private_margin_repaid_infos = privateGetSpotV3PrivateMarginRepaidInfos = Entry('spot/v3/private/margin-repaid-infos', 'private', 'GET', {'cost': 10})
    private_get_spot_v3_private_margin_ltv = privateGetSpotV3PrivateMarginLtv = Entry('spot/v3/private/margin-ltv', 'private', 'GET', {'cost': 10})
    private_get_asset_v3_private_transfer_inter_transfer_list_query = privateGetAssetV3PrivateTransferInterTransferListQuery = Entry('asset/v3/private/transfer/inter-transfer/list/query', 'private', 'GET', {'cost': 50})
    private_get_asset_v3_private_transfer_sub_member_list_query = privateGetAssetV3PrivateTransferSubMemberListQuery = Entry('asset/v3/private/transfer/sub-member/list/query', 'private', 'GET', {'cost': 50})
    private_get_asset_v3_private_transfer_sub_member_transfer_list_query = privateGetAssetV3PrivateTransferSubMemberTransferListQuery = Entry('asset/v3/private/transfer/sub-member-transfer/list/query', 'private', 'GET', {'cost': 50})
    private_get_asset_v3_private_transfer_universal_transfer_list_query = privateGetAssetV3PrivateTransferUniversalTransferListQuery = Entry('asset/v3/private/transfer/universal-transfer/list/query', 'private', 'GET', {'cost': 25})
    private_get_asset_v3_private_coin_info_query = privateGetAssetV3PrivateCoinInfoQuery = Entry('asset/v3/private/coin-info/query', 'private', 'GET', {'cost': 25})
    private_get_asset_v3_private_deposit_address_query = privateGetAssetV3PrivateDepositAddressQuery = Entry('asset/v3/private/deposit/address/query', 'private', 'GET', {'cost': 10})
    private_get_contract_v3_private_copytrading_order_list = privateGetContractV3PrivateCopytradingOrderList = Entry('contract/v3/private/copytrading/order/list', 'private', 'GET', {'cost': 30})
    private_get_contract_v3_private_copytrading_position_list = privateGetContractV3PrivateCopytradingPositionList = Entry('contract/v3/private/copytrading/position/list', 'private', 'GET', {'cost': 40})
    private_get_contract_v3_private_copytrading_wallet_balance = privateGetContractV3PrivateCopytradingWalletBalance = Entry('contract/v3/private/copytrading/wallet/balance', 'private', 'GET', {'cost': 25})
    private_get_contract_v3_private_position_limit_info = privateGetContractV3PrivatePositionLimitInfo = Entry('contract/v3/private/position/limit-info', 'private', 'GET', {'cost': 25})
    private_get_contract_v3_private_order_unfilled_orders = privateGetContractV3PrivateOrderUnfilledOrders = Entry('contract/v3/private/order/unfilled-orders', 'private', 'GET', {'cost': 1})
    private_get_contract_v3_private_order_list = privateGetContractV3PrivateOrderList = Entry('contract/v3/private/order/list', 'private', 'GET', {'cost': 1})
    private_get_contract_v3_private_position_list = privateGetContractV3PrivatePositionList = Entry('contract/v3/private/position/list', 'private', 'GET', {'cost': 1})
    private_get_contract_v3_private_execution_list = privateGetContractV3PrivateExecutionList = Entry('contract/v3/private/execution/list', 'private', 'GET', {'cost': 1})
    private_get_contract_v3_private_position_closed_pnl = privateGetContractV3PrivatePositionClosedPnl = Entry('contract/v3/private/position/closed-pnl', 'private', 'GET', {'cost': 1})
    private_get_contract_v3_private_account_wallet_balance = privateGetContractV3PrivateAccountWalletBalance = Entry('contract/v3/private/account/wallet/balance', 'private', 'GET', {'cost': 1})
    private_get_contract_v3_private_account_fee_rate = privateGetContractV3PrivateAccountFeeRate = Entry('contract/v3/private/account/fee-rate', 'private', 'GET', {'cost': 1})
    private_get_contract_v3_private_account_wallet_fund_records = privateGetContractV3PrivateAccountWalletFundRecords = Entry('contract/v3/private/account/wallet/fund-records', 'private', 'GET', {'cost': 1})
    private_get_unified_v3_private_order_unfilled_orders = privateGetUnifiedV3PrivateOrderUnfilledOrders = Entry('unified/v3/private/order/unfilled-orders', 'private', 'GET', {'cost': 1})
    private_get_unified_v3_private_order_list = privateGetUnifiedV3PrivateOrderList = Entry('unified/v3/private/order/list', 'private', 'GET', {'cost': 1})
    private_get_unified_v3_private_position_list = privateGetUnifiedV3PrivatePositionList = Entry('unified/v3/private/position/list', 'private', 'GET', {'cost': 1})
    private_get_unified_v3_private_execution_list = privateGetUnifiedV3PrivateExecutionList = Entry('unified/v3/private/execution/list', 'private', 'GET', {'cost': 1})
    private_get_unified_v3_private_delivery_record = privateGetUnifiedV3PrivateDeliveryRecord = Entry('unified/v3/private/delivery-record', 'private', 'GET', {'cost': 1})
    private_get_unified_v3_private_settlement_record = privateGetUnifiedV3PrivateSettlementRecord = Entry('unified/v3/private/settlement-record', 'private', 'GET', {'cost': 1})
    private_get_unified_v3_private_account_wallet_balance = privateGetUnifiedV3PrivateAccountWalletBalance = Entry('unified/v3/private/account/wallet/balance', 'private', 'GET', {'cost': 1})
    private_get_unified_v3_private_account_transaction_log = privateGetUnifiedV3PrivateAccountTransactionLog = Entry('unified/v3/private/account/transaction-log', 'private', 'GET', {'cost': 1})
    private_get_unified_v3_private_account_borrow_history = privateGetUnifiedV3PrivateAccountBorrowHistory = Entry('unified/v3/private/account/borrow-history', 'private', 'GET', {'cost': 1})
    private_get_unified_v3_private_account_borrow_rate = privateGetUnifiedV3PrivateAccountBorrowRate = Entry('unified/v3/private/account/borrow-rate', 'private', 'GET', {'cost': 1})
    private_get_unified_v3_private_account_info = privateGetUnifiedV3PrivateAccountInfo = Entry('unified/v3/private/account/info', 'private', 'GET', {'cost': 1})
    private_get_user_v3_private_frozen_sub_member = privateGetUserV3PrivateFrozenSubMember = Entry('user/v3/private/frozen-sub-member', 'private', 'GET', {'cost': 10})
    private_get_user_v3_private_query_sub_members = privateGetUserV3PrivateQuerySubMembers = Entry('user/v3/private/query-sub-members', 'private', 'GET', {'cost': 5})
    private_get_user_v3_private_query_api = privateGetUserV3PrivateQueryApi = Entry('user/v3/private/query-api', 'private', 'GET', {'cost': 5})
    private_get_user_v3_private_get_member_type = privateGetUserV3PrivateGetMemberType = Entry('user/v3/private/get-member-type', 'private', 'GET', {'cost': 1})
    private_get_asset_v3_private_transfer_transfer_coin_list_query = privateGetAssetV3PrivateTransferTransferCoinListQuery = Entry('asset/v3/private/transfer/transfer-coin/list/query', 'private', 'GET', {'cost': 50})
    private_get_asset_v3_private_transfer_account_coin_balance_query = privateGetAssetV3PrivateTransferAccountCoinBalanceQuery = Entry('asset/v3/private/transfer/account-coin/balance/query', 'private', 'GET', {'cost': 50})
    private_get_asset_v3_private_transfer_account_coins_balance_query = privateGetAssetV3PrivateTransferAccountCoinsBalanceQuery = Entry('asset/v3/private/transfer/account-coins/balance/query', 'private', 'GET', {'cost': 25})
    private_get_asset_v3_private_transfer_asset_info_query = privateGetAssetV3PrivateTransferAssetInfoQuery = Entry('asset/v3/private/transfer/asset-info/query', 'private', 'GET', {'cost': 50})
    private_get_asset_v3_public_deposit_allowed_deposit_list_query = privateGetAssetV3PublicDepositAllowedDepositListQuery = Entry('asset/v3/public/deposit/allowed-deposit-list/query', 'private', 'GET', {'cost': 0.17})
    private_get_asset_v3_private_deposit_record_query = privateGetAssetV3PrivateDepositRecordQuery = Entry('asset/v3/private/deposit/record/query', 'private', 'GET', {'cost': 10})
    private_get_asset_v3_private_withdraw_record_query = privateGetAssetV3PrivateWithdrawRecordQuery = Entry('asset/v3/private/withdraw/record/query', 'private', 'GET', {'cost': 10})
    private_get_v5_order_realtime = privateGetV5OrderRealtime = Entry('v5/order/realtime', 'private', 'GET', {'cost': 5})
    private_get_v5_order_history = privateGetV5OrderHistory = Entry('v5/order/history', 'private', 'GET', {'cost': 5})
    private_get_v5_order_spot_borrow_check = privateGetV5OrderSpotBorrowCheck = Entry('v5/order/spot-borrow-check', 'private', 'GET', {'cost': 1})
    private_get_v5_position_list = privateGetV5PositionList = Entry('v5/position/list', 'private', 'GET', {'cost': 5})
    private_get_v5_execution_list = privateGetV5ExecutionList = Entry('v5/execution/list', 'private', 'GET', {'cost': 5})
    private_get_v5_position_closed_pnl = privateGetV5PositionClosedPnl = Entry('v5/position/closed-pnl', 'private', 'GET', {'cost': 5})
    private_get_v5_position_move_history = privateGetV5PositionMoveHistory = Entry('v5/position/move-history', 'private', 'GET', {'cost': 5})
    private_get_v5_pre_upgrade_order_history = privateGetV5PreUpgradeOrderHistory = Entry('v5/pre-upgrade/order/history', 'private', 'GET', {'cost': 5})
    private_get_v5_pre_upgrade_execution_list = privateGetV5PreUpgradeExecutionList = Entry('v5/pre-upgrade/execution/list', 'private', 'GET', {'cost': 5})
    private_get_v5_pre_upgrade_position_closed_pnl = privateGetV5PreUpgradePositionClosedPnl = Entry('v5/pre-upgrade/position/closed-pnl', 'private', 'GET', {'cost': 5})
    private_get_v5_pre_upgrade_account_transaction_log = privateGetV5PreUpgradeAccountTransactionLog = Entry('v5/pre-upgrade/account/transaction-log', 'private', 'GET', {'cost': 5})
    private_get_v5_pre_upgrade_asset_delivery_record = privateGetV5PreUpgradeAssetDeliveryRecord = Entry('v5/pre-upgrade/asset/delivery-record', 'private', 'GET', {'cost': 5})
    private_get_v5_pre_upgrade_asset_settlement_record = privateGetV5PreUpgradeAssetSettlementRecord = Entry('v5/pre-upgrade/asset/settlement-record', 'private', 'GET', {'cost': 5})
    private_get_v5_account_wallet_balance = privateGetV5AccountWalletBalance = Entry('v5/account/wallet-balance', 'private', 'GET', {'cost': 1})
    private_get_v5_account_borrow_history = privateGetV5AccountBorrowHistory = Entry('v5/account/borrow-history', 'private', 'GET', {'cost': 1})
    private_get_v5_account_collateral_info = privateGetV5AccountCollateralInfo = Entry('v5/account/collateral-info', 'private', 'GET', {'cost': 1})
    private_get_v5_asset_coin_greeks = privateGetV5AssetCoinGreeks = Entry('v5/asset/coin-greeks', 'private', 'GET', {'cost': 1})
    private_get_v5_account_fee_rate = privateGetV5AccountFeeRate = Entry('v5/account/fee-rate', 'private', 'GET', {'cost': 10})
    private_get_v5_account_info = privateGetV5AccountInfo = Entry('v5/account/info', 'private', 'GET', {'cost': 5})
    private_get_v5_account_transaction_log = privateGetV5AccountTransactionLog = Entry('v5/account/transaction-log', 'private', 'GET', {'cost': 1})
    private_get_v5_account_contract_transaction_log = privateGetV5AccountContractTransactionLog = Entry('v5/account/contract-transaction-log', 'private', 'GET', {'cost': 1})
    private_get_v5_account_smp_group = privateGetV5AccountSmpGroup = Entry('v5/account/smp-group', 'private', 'GET', {'cost': 1})
    private_get_v5_account_mmp_state = privateGetV5AccountMmpState = Entry('v5/account/mmp-state', 'private', 'GET', {'cost': 5})
    private_get_v5_asset_exchange_order_record = privateGetV5AssetExchangeOrderRecord = Entry('v5/asset/exchange/order-record', 'private', 'GET', {'cost': 5})
    private_get_v5_asset_delivery_record = privateGetV5AssetDeliveryRecord = Entry('v5/asset/delivery-record', 'private', 'GET', {'cost': 5})
    private_get_v5_asset_settlement_record = privateGetV5AssetSettlementRecord = Entry('v5/asset/settlement-record', 'private', 'GET', {'cost': 5})
    private_get_v5_asset_transfer_query_asset_info = privateGetV5AssetTransferQueryAssetInfo = Entry('v5/asset/transfer/query-asset-info', 'private', 'GET', {'cost': 50})
    private_get_v5_asset_transfer_query_account_coins_balance = privateGetV5AssetTransferQueryAccountCoinsBalance = Entry('v5/asset/transfer/query-account-coins-balance', 'private', 'GET', {'cost': 25})
    private_get_v5_asset_transfer_query_account_coin_balance = privateGetV5AssetTransferQueryAccountCoinBalance = Entry('v5/asset/transfer/query-account-coin-balance', 'private', 'GET', {'cost': 50})
    private_get_v5_asset_transfer_query_transfer_coin_list = privateGetV5AssetTransferQueryTransferCoinList = Entry('v5/asset/transfer/query-transfer-coin-list', 'private', 'GET', {'cost': 50})
    private_get_v5_asset_transfer_query_inter_transfer_list = privateGetV5AssetTransferQueryInterTransferList = Entry('v5/asset/transfer/query-inter-transfer-list', 'private', 'GET', {'cost': 50})
    private_get_v5_asset_transfer_query_sub_member_list = privateGetV5AssetTransferQuerySubMemberList = Entry('v5/asset/transfer/query-sub-member-list', 'private', 'GET', {'cost': 50})
    private_get_v5_asset_transfer_query_universal_transfer_list = privateGetV5AssetTransferQueryUniversalTransferList = Entry('v5/asset/transfer/query-universal-transfer-list', 'private', 'GET', {'cost': 25})
    private_get_v5_asset_deposit_query_allowed_list = privateGetV5AssetDepositQueryAllowedList = Entry('v5/asset/deposit/query-allowed-list', 'private', 'GET', {'cost': 5})
    private_get_v5_asset_deposit_query_record = privateGetV5AssetDepositQueryRecord = Entry('v5/asset/deposit/query-record', 'private', 'GET', {'cost': 10})
    private_get_v5_asset_deposit_query_sub_member_record = privateGetV5AssetDepositQuerySubMemberRecord = Entry('v5/asset/deposit/query-sub-member-record', 'private', 'GET', {'cost': 10})
    private_get_v5_asset_deposit_query_internal_record = privateGetV5AssetDepositQueryInternalRecord = Entry('v5/asset/deposit/query-internal-record', 'private', 'GET', {'cost': 5})
    private_get_v5_asset_deposit_query_address = privateGetV5AssetDepositQueryAddress = Entry('v5/asset/deposit/query-address', 'private', 'GET', {'cost': 10})
    private_get_v5_asset_deposit_query_sub_member_address = privateGetV5AssetDepositQuerySubMemberAddress = Entry('v5/asset/deposit/query-sub-member-address', 'private', 'GET', {'cost': 10})
    private_get_v5_asset_coin_query_info = privateGetV5AssetCoinQueryInfo = Entry('v5/asset/coin/query-info', 'private', 'GET', {'cost': 28})
    private_get_v5_asset_withdraw_query_record = privateGetV5AssetWithdrawQueryRecord = Entry('v5/asset/withdraw/query-record', 'private', 'GET', {'cost': 10})
    private_get_v5_asset_withdraw_withdrawable_amount = privateGetV5AssetWithdrawWithdrawableAmount = Entry('v5/asset/withdraw/withdrawable-amount', 'private', 'GET', {'cost': 5})
    private_get_v5_asset_withdraw_vasp_list = privateGetV5AssetWithdrawVaspList = Entry('v5/asset/withdraw/vasp/list', 'private', 'GET', {'cost': 5})
    private_get_v5_user_query_sub_members = privateGetV5UserQuerySubMembers = Entry('v5/user/query-sub-members', 'private', 'GET', {'cost': 5})
    private_get_v5_user_query_api = privateGetV5UserQueryApi = Entry('v5/user/query-api', 'private', 'GET', {'cost': 5})
    private_get_v5_user_sub_apikeys = privateGetV5UserSubApikeys = Entry('v5/user/sub-apikeys', 'private', 'GET', {'cost': 5})
    private_get_v5_user_get_member_type = privateGetV5UserGetMemberType = Entry('v5/user/get-member-type', 'private', 'GET', {'cost': 5})
    private_get_v5_user_aff_customer_info = privateGetV5UserAffCustomerInfo = Entry('v5/user/aff-customer-info', 'private', 'GET', {'cost': 5})
    private_get_v5_user_del_submember = privateGetV5UserDelSubmember = Entry('v5/user/del-submember', 'private', 'GET', {'cost': 5})
    private_get_v5_user_submembers = privateGetV5UserSubmembers = Entry('v5/user/submembers', 'private', 'GET', {'cost': 5})
    private_get_v5_spot_lever_token_order_record = privateGetV5SpotLeverTokenOrderRecord = Entry('v5/spot-lever-token/order-record', 'private', 'GET', {'cost': 1})
    private_get_v5_spot_margin_trade_state = privateGetV5SpotMarginTradeState = Entry('v5/spot-margin-trade/state', 'private', 'GET', {'cost': 5})
    private_get_v5_spot_cross_margin_trade_loan_info = privateGetV5SpotCrossMarginTradeLoanInfo = Entry('v5/spot-cross-margin-trade/loan-info', 'private', 'GET', {'cost': 1})
    private_get_v5_spot_cross_margin_trade_account = privateGetV5SpotCrossMarginTradeAccount = Entry('v5/spot-cross-margin-trade/account', 'private', 'GET', {'cost': 1})
    private_get_v5_spot_cross_margin_trade_orders = privateGetV5SpotCrossMarginTradeOrders = Entry('v5/spot-cross-margin-trade/orders', 'private', 'GET', {'cost': 1})
    private_get_v5_spot_cross_margin_trade_repay_history = privateGetV5SpotCrossMarginTradeRepayHistory = Entry('v5/spot-cross-margin-trade/repay-history', 'private', 'GET', {'cost': 1})
    private_get_v5_ins_loan_product_infos = privateGetV5InsLoanProductInfos = Entry('v5/ins-loan/product-infos', 'private', 'GET', {'cost': 5})
    private_get_v5_ins_loan_ensure_tokens_convert = privateGetV5InsLoanEnsureTokensConvert = Entry('v5/ins-loan/ensure-tokens-convert', 'private', 'GET', {'cost': 5})
    private_get_v5_ins_loan_loan_order = privateGetV5InsLoanLoanOrder = Entry('v5/ins-loan/loan-order', 'private', 'GET', {'cost': 5})
    private_get_v5_ins_loan_repaid_history = privateGetV5InsLoanRepaidHistory = Entry('v5/ins-loan/repaid-history', 'private', 'GET', {'cost': 5})
    private_get_v5_ins_loan_ltv_convert = privateGetV5InsLoanLtvConvert = Entry('v5/ins-loan/ltv-convert', 'private', 'GET', {'cost': 5})
    private_get_v5_lending_info = privateGetV5LendingInfo = Entry('v5/lending/info', 'private', 'GET', {'cost': 5})
    private_get_v5_lending_history_order = privateGetV5LendingHistoryOrder = Entry('v5/lending/history-order', 'private', 'GET', {'cost': 5})
    private_get_v5_lending_account = privateGetV5LendingAccount = Entry('v5/lending/account', 'private', 'GET', {'cost': 5})
    private_get_v5_broker_earning_record = privateGetV5BrokerEarningRecord = Entry('v5/broker/earning-record', 'private', 'GET', {'cost': 5})
    private_get_v5_broker_earnings_info = privateGetV5BrokerEarningsInfo = Entry('v5/broker/earnings-info', 'private', 'GET', {'cost': 5})
    private_get_v5_broker_account_info = privateGetV5BrokerAccountInfo = Entry('v5/broker/account-info', 'private', 'GET', {'cost': 5})
    private_get_v5_broker_asset_query_sub_member_deposit_record = privateGetV5BrokerAssetQuerySubMemberDepositRecord = Entry('v5/broker/asset/query-sub-member-deposit-record', 'private', 'GET', {'cost': 10})
    private_post_option_usdc_openapi_private_v1_place_order = privatePostOptionUsdcOpenapiPrivateV1PlaceOrder = Entry('option/usdc/openapi/private/v1/place-order', 'private', 'POST', {'cost': 2.5})
    private_post_option_usdc_openapi_private_v1_replace_order = privatePostOptionUsdcOpenapiPrivateV1ReplaceOrder = Entry('option/usdc/openapi/private/v1/replace-order', 'private', 'POST', {'cost': 2.5})
    private_post_option_usdc_openapi_private_v1_cancel_order = privatePostOptionUsdcOpenapiPrivateV1CancelOrder = Entry('option/usdc/openapi/private/v1/cancel-order', 'private', 'POST', {'cost': 2.5})
    private_post_option_usdc_openapi_private_v1_cancel_all = privatePostOptionUsdcOpenapiPrivateV1CancelAll = Entry('option/usdc/openapi/private/v1/cancel-all', 'private', 'POST', {'cost': 2.5})
    private_post_option_usdc_openapi_private_v1_query_active_orders = privatePostOptionUsdcOpenapiPrivateV1QueryActiveOrders = Entry('option/usdc/openapi/private/v1/query-active-orders', 'private', 'POST', {'cost': 2.5})
    private_post_option_usdc_openapi_private_v1_query_order_history = privatePostOptionUsdcOpenapiPrivateV1QueryOrderHistory = Entry('option/usdc/openapi/private/v1/query-order-history', 'private', 'POST', {'cost': 2.5})
    private_post_option_usdc_openapi_private_v1_execution_list = privatePostOptionUsdcOpenapiPrivateV1ExecutionList = Entry('option/usdc/openapi/private/v1/execution-list', 'private', 'POST', {'cost': 2.5})
    private_post_option_usdc_openapi_private_v1_query_position = privatePostOptionUsdcOpenapiPrivateV1QueryPosition = Entry('option/usdc/openapi/private/v1/query-position', 'private', 'POST', {'cost': 2.5})
    private_post_perpetual_usdc_openapi_private_v1_place_order = privatePostPerpetualUsdcOpenapiPrivateV1PlaceOrder = Entry('perpetual/usdc/openapi/private/v1/place-order', 'private', 'POST', {'cost': 2.5})
    private_post_perpetual_usdc_openapi_private_v1_replace_order = privatePostPerpetualUsdcOpenapiPrivateV1ReplaceOrder = Entry('perpetual/usdc/openapi/private/v1/replace-order', 'private', 'POST', {'cost': 2.5})
    private_post_perpetual_usdc_openapi_private_v1_cancel_order = privatePostPerpetualUsdcOpenapiPrivateV1CancelOrder = Entry('perpetual/usdc/openapi/private/v1/cancel-order', 'private', 'POST', {'cost': 2.5})
    private_post_perpetual_usdc_openapi_private_v1_cancel_all = privatePostPerpetualUsdcOpenapiPrivateV1CancelAll = Entry('perpetual/usdc/openapi/private/v1/cancel-all', 'private', 'POST', {'cost': 2.5})
    private_post_perpetual_usdc_openapi_private_v1_position_leverage_save = privatePostPerpetualUsdcOpenapiPrivateV1PositionLeverageSave = Entry('perpetual/usdc/openapi/private/v1/position/leverage/save', 'private', 'POST', {'cost': 2.5})
    private_post_spot_v3_private_order = privatePostSpotV3PrivateOrder = Entry('spot/v3/private/order', 'private', 'POST', {'cost': 2.5})
    private_post_spot_v3_private_cancel_order = privatePostSpotV3PrivateCancelOrder = Entry('spot/v3/private/cancel-order', 'private', 'POST', {'cost': 2.5})
    private_post_spot_v3_private_cancel_orders = privatePostSpotV3PrivateCancelOrders = Entry('spot/v3/private/cancel-orders', 'private', 'POST', {'cost': 2.5})
    private_post_spot_v3_private_cancel_orders_by_ids = privatePostSpotV3PrivateCancelOrdersByIds = Entry('spot/v3/private/cancel-orders-by-ids', 'private', 'POST', {'cost': 2.5})
    private_post_spot_v3_private_purchase = privatePostSpotV3PrivatePurchase = Entry('spot/v3/private/purchase', 'private', 'POST', {'cost': 2.5})
    private_post_spot_v3_private_redeem = privatePostSpotV3PrivateRedeem = Entry('spot/v3/private/redeem', 'private', 'POST', {'cost': 2.5})
    private_post_spot_v3_private_cross_margin_loan = privatePostSpotV3PrivateCrossMarginLoan = Entry('spot/v3/private/cross-margin-loan', 'private', 'POST', {'cost': 10})
    private_post_spot_v3_private_cross_margin_repay = privatePostSpotV3PrivateCrossMarginRepay = Entry('spot/v3/private/cross-margin-repay', 'private', 'POST', {'cost': 10})
    private_post_asset_v3_private_transfer_inter_transfer = privatePostAssetV3PrivateTransferInterTransfer = Entry('asset/v3/private/transfer/inter-transfer', 'private', 'POST', {'cost': 150})
    private_post_asset_v3_private_withdraw_create = privatePostAssetV3PrivateWithdrawCreate = Entry('asset/v3/private/withdraw/create', 'private', 'POST', {'cost': 300})
    private_post_asset_v3_private_withdraw_cancel = privatePostAssetV3PrivateWithdrawCancel = Entry('asset/v3/private/withdraw/cancel', 'private', 'POST', {'cost': 50})
    private_post_asset_v3_private_transfer_sub_member_transfer = privatePostAssetV3PrivateTransferSubMemberTransfer = Entry('asset/v3/private/transfer/sub-member-transfer', 'private', 'POST', {'cost': 150})
    private_post_asset_v3_private_transfer_transfer_sub_member_save = privatePostAssetV3PrivateTransferTransferSubMemberSave = Entry('asset/v3/private/transfer/transfer-sub-member-save', 'private', 'POST', {'cost': 150})
    private_post_asset_v3_private_transfer_universal_transfer = privatePostAssetV3PrivateTransferUniversalTransfer = Entry('asset/v3/private/transfer/universal-transfer', 'private', 'POST', {'cost': 10})
    private_post_user_v3_private_create_sub_member = privatePostUserV3PrivateCreateSubMember = Entry('user/v3/private/create-sub-member', 'private', 'POST', {'cost': 10})
    private_post_user_v3_private_create_sub_api = privatePostUserV3PrivateCreateSubApi = Entry('user/v3/private/create-sub-api', 'private', 'POST', {'cost': 10})
    private_post_user_v3_private_update_api = privatePostUserV3PrivateUpdateApi = Entry('user/v3/private/update-api', 'private', 'POST', {'cost': 10})
    private_post_user_v3_private_delete_api = privatePostUserV3PrivateDeleteApi = Entry('user/v3/private/delete-api', 'private', 'POST', {'cost': 10})
    private_post_user_v3_private_update_sub_api = privatePostUserV3PrivateUpdateSubApi = Entry('user/v3/private/update-sub-api', 'private', 'POST', {'cost': 10})
    private_post_user_v3_private_delete_sub_api = privatePostUserV3PrivateDeleteSubApi = Entry('user/v3/private/delete-sub-api', 'private', 'POST', {'cost': 10})
    private_post_contract_v3_private_copytrading_order_create = privatePostContractV3PrivateCopytradingOrderCreate = Entry('contract/v3/private/copytrading/order/create', 'private', 'POST', {'cost': 30})
    private_post_contract_v3_private_copytrading_order_cancel = privatePostContractV3PrivateCopytradingOrderCancel = Entry('contract/v3/private/copytrading/order/cancel', 'private', 'POST', {'cost': 30})
    private_post_contract_v3_private_copytrading_order_close = privatePostContractV3PrivateCopytradingOrderClose = Entry('contract/v3/private/copytrading/order/close', 'private', 'POST', {'cost': 30})
    private_post_contract_v3_private_copytrading_position_close = privatePostContractV3PrivateCopytradingPositionClose = Entry('contract/v3/private/copytrading/position/close', 'private', 'POST', {'cost': 40})
    private_post_contract_v3_private_copytrading_position_set_leverage = privatePostContractV3PrivateCopytradingPositionSetLeverage = Entry('contract/v3/private/copytrading/position/set-leverage', 'private', 'POST', {'cost': 40})
    private_post_contract_v3_private_copytrading_wallet_transfer = privatePostContractV3PrivateCopytradingWalletTransfer = Entry('contract/v3/private/copytrading/wallet/transfer', 'private', 'POST', {'cost': 25})
    private_post_contract_v3_private_copytrading_order_trading_stop = privatePostContractV3PrivateCopytradingOrderTradingStop = Entry('contract/v3/private/copytrading/order/trading-stop', 'private', 'POST', {'cost': 2.5})
    private_post_contract_v3_private_order_create = privatePostContractV3PrivateOrderCreate = Entry('contract/v3/private/order/create', 'private', 'POST', {'cost': 1})
    private_post_contract_v3_private_order_cancel = privatePostContractV3PrivateOrderCancel = Entry('contract/v3/private/order/cancel', 'private', 'POST', {'cost': 1})
    private_post_contract_v3_private_order_cancel_all = privatePostContractV3PrivateOrderCancelAll = Entry('contract/v3/private/order/cancel-all', 'private', 'POST', {'cost': 1})
    private_post_contract_v3_private_order_replace = privatePostContractV3PrivateOrderReplace = Entry('contract/v3/private/order/replace', 'private', 'POST', {'cost': 1})
    private_post_contract_v3_private_position_set_auto_add_margin = privatePostContractV3PrivatePositionSetAutoAddMargin = Entry('contract/v3/private/position/set-auto-add-margin', 'private', 'POST', {'cost': 1})
    private_post_contract_v3_private_position_switch_isolated = privatePostContractV3PrivatePositionSwitchIsolated = Entry('contract/v3/private/position/switch-isolated', 'private', 'POST', {'cost': 1})
    private_post_contract_v3_private_position_switch_mode = privatePostContractV3PrivatePositionSwitchMode = Entry('contract/v3/private/position/switch-mode', 'private', 'POST', {'cost': 1})
    private_post_contract_v3_private_position_switch_tpsl_mode = privatePostContractV3PrivatePositionSwitchTpslMode = Entry('contract/v3/private/position/switch-tpsl-mode', 'private', 'POST', {'cost': 1})
    private_post_contract_v3_private_position_set_leverage = privatePostContractV3PrivatePositionSetLeverage = Entry('contract/v3/private/position/set-leverage', 'private', 'POST', {'cost': 1})
    private_post_contract_v3_private_position_trading_stop = privatePostContractV3PrivatePositionTradingStop = Entry('contract/v3/private/position/trading-stop', 'private', 'POST', {'cost': 1})
    private_post_contract_v3_private_position_set_risk_limit = privatePostContractV3PrivatePositionSetRiskLimit = Entry('contract/v3/private/position/set-risk-limit', 'private', 'POST', {'cost': 1})
    private_post_contract_v3_private_account_setmarginmode = privatePostContractV3PrivateAccountSetMarginMode = Entry('contract/v3/private/account/setMarginMode', 'private', 'POST', {'cost': 1})
    private_post_unified_v3_private_order_create = privatePostUnifiedV3PrivateOrderCreate = Entry('unified/v3/private/order/create', 'private', 'POST', {'cost': 30})
    private_post_unified_v3_private_order_replace = privatePostUnifiedV3PrivateOrderReplace = Entry('unified/v3/private/order/replace', 'private', 'POST', {'cost': 30})
    private_post_unified_v3_private_order_cancel = privatePostUnifiedV3PrivateOrderCancel = Entry('unified/v3/private/order/cancel', 'private', 'POST', {'cost': 30})
    private_post_unified_v3_private_order_create_batch = privatePostUnifiedV3PrivateOrderCreateBatch = Entry('unified/v3/private/order/create-batch', 'private', 'POST', {'cost': 30})
    private_post_unified_v3_private_order_replace_batch = privatePostUnifiedV3PrivateOrderReplaceBatch = Entry('unified/v3/private/order/replace-batch', 'private', 'POST', {'cost': 30})
    private_post_unified_v3_private_order_cancel_batch = privatePostUnifiedV3PrivateOrderCancelBatch = Entry('unified/v3/private/order/cancel-batch', 'private', 'POST', {'cost': 30})
    private_post_unified_v3_private_order_cancel_all = privatePostUnifiedV3PrivateOrderCancelAll = Entry('unified/v3/private/order/cancel-all', 'private', 'POST', {'cost': 30})
    private_post_unified_v3_private_position_set_leverage = privatePostUnifiedV3PrivatePositionSetLeverage = Entry('unified/v3/private/position/set-leverage', 'private', 'POST', {'cost': 2.5})
    private_post_unified_v3_private_position_tpsl_switch_mode = privatePostUnifiedV3PrivatePositionTpslSwitchMode = Entry('unified/v3/private/position/tpsl/switch-mode', 'private', 'POST', {'cost': 2.5})
    private_post_unified_v3_private_position_set_risk_limit = privatePostUnifiedV3PrivatePositionSetRiskLimit = Entry('unified/v3/private/position/set-risk-limit', 'private', 'POST', {'cost': 2.5})
    private_post_unified_v3_private_position_trading_stop = privatePostUnifiedV3PrivatePositionTradingStop = Entry('unified/v3/private/position/trading-stop', 'private', 'POST', {'cost': 2.5})
    private_post_unified_v3_private_account_upgrade_unified_account = privatePostUnifiedV3PrivateAccountUpgradeUnifiedAccount = Entry('unified/v3/private/account/upgrade-unified-account', 'private', 'POST', {'cost': 2.5})
    private_post_unified_v3_private_account_setmarginmode = privatePostUnifiedV3PrivateAccountSetMarginMode = Entry('unified/v3/private/account/setMarginMode', 'private', 'POST', {'cost': 2.5})
    private_post_fht_compliance_tax_v3_private_registertime = privatePostFhtComplianceTaxV3PrivateRegistertime = Entry('fht/compliance/tax/v3/private/registertime', 'private', 'POST', {'cost': 50})
    private_post_fht_compliance_tax_v3_private_create = privatePostFhtComplianceTaxV3PrivateCreate = Entry('fht/compliance/tax/v3/private/create', 'private', 'POST', {'cost': 50})
    private_post_fht_compliance_tax_v3_private_status = privatePostFhtComplianceTaxV3PrivateStatus = Entry('fht/compliance/tax/v3/private/status', 'private', 'POST', {'cost': 50})
    private_post_fht_compliance_tax_v3_private_url = privatePostFhtComplianceTaxV3PrivateUrl = Entry('fht/compliance/tax/v3/private/url', 'private', 'POST', {'cost': 50})
    private_post_v5_order_create = privatePostV5OrderCreate = Entry('v5/order/create', 'private', 'POST', {'cost': 2.5})
    private_post_v5_order_amend = privatePostV5OrderAmend = Entry('v5/order/amend', 'private', 'POST', {'cost': 5})
    private_post_v5_order_cancel = privatePostV5OrderCancel = Entry('v5/order/cancel', 'private', 'POST', {'cost': 2.5})
    private_post_v5_order_cancel_all = privatePostV5OrderCancelAll = Entry('v5/order/cancel-all', 'private', 'POST', {'cost': 50})
    private_post_v5_order_create_batch = privatePostV5OrderCreateBatch = Entry('v5/order/create-batch', 'private', 'POST', {'cost': 5})
    private_post_v5_order_amend_batch = privatePostV5OrderAmendBatch = Entry('v5/order/amend-batch', 'private', 'POST', {'cost': 5})
    private_post_v5_order_cancel_batch = privatePostV5OrderCancelBatch = Entry('v5/order/cancel-batch', 'private', 'POST', {'cost': 5})
    private_post_v5_order_disconnected_cancel_all = privatePostV5OrderDisconnectedCancelAll = Entry('v5/order/disconnected-cancel-all', 'private', 'POST', {'cost': 5})
    private_post_v5_position_set_leverage = privatePostV5PositionSetLeverage = Entry('v5/position/set-leverage', 'private', 'POST', {'cost': 5})
    private_post_v5_position_switch_isolated = privatePostV5PositionSwitchIsolated = Entry('v5/position/switch-isolated', 'private', 'POST', {'cost': 5})
    private_post_v5_position_set_tpsl_mode = privatePostV5PositionSetTpslMode = Entry('v5/position/set-tpsl-mode', 'private', 'POST', {'cost': 5})
    private_post_v5_position_switch_mode = privatePostV5PositionSwitchMode = Entry('v5/position/switch-mode', 'private', 'POST', {'cost': 5})
    private_post_v5_position_set_risk_limit = privatePostV5PositionSetRiskLimit = Entry('v5/position/set-risk-limit', 'private', 'POST', {'cost': 5})
    private_post_v5_position_trading_stop = privatePostV5PositionTradingStop = Entry('v5/position/trading-stop', 'private', 'POST', {'cost': 5})
    private_post_v5_position_set_auto_add_margin = privatePostV5PositionSetAutoAddMargin = Entry('v5/position/set-auto-add-margin', 'private', 'POST', {'cost': 5})
    private_post_v5_position_add_margin = privatePostV5PositionAddMargin = Entry('v5/position/add-margin', 'private', 'POST', {'cost': 5})
    private_post_v5_position_move_positions = privatePostV5PositionMovePositions = Entry('v5/position/move-positions', 'private', 'POST', {'cost': 5})
    private_post_v5_position_confirm_pending_mmr = privatePostV5PositionConfirmPendingMmr = Entry('v5/position/confirm-pending-mmr', 'private', 'POST', {'cost': 5})
    private_post_v5_account_upgrade_to_uta = privatePostV5AccountUpgradeToUta = Entry('v5/account/upgrade-to-uta', 'private', 'POST', {'cost': 5})
    private_post_v5_account_quick_repayment = privatePostV5AccountQuickRepayment = Entry('v5/account/quick-repayment', 'private', 'POST', {'cost': 5})
    private_post_v5_account_set_margin_mode = privatePostV5AccountSetMarginMode = Entry('v5/account/set-margin-mode', 'private', 'POST', {'cost': 5})
    private_post_v5_account_set_hedging_mode = privatePostV5AccountSetHedgingMode = Entry('v5/account/set-hedging-mode', 'private', 'POST', {'cost': 5})
    private_post_v5_account_mmp_modify = privatePostV5AccountMmpModify = Entry('v5/account/mmp-modify', 'private', 'POST', {'cost': 5})
    private_post_v5_account_mmp_reset = privatePostV5AccountMmpReset = Entry('v5/account/mmp-reset', 'private', 'POST', {'cost': 5})
    private_post_v5_asset_transfer_inter_transfer = privatePostV5AssetTransferInterTransfer = Entry('v5/asset/transfer/inter-transfer', 'private', 'POST', {'cost': 50})
    private_post_v5_asset_transfer_save_transfer_sub_member = privatePostV5AssetTransferSaveTransferSubMember = Entry('v5/asset/transfer/save-transfer-sub-member', 'private', 'POST', {'cost': 150})
    private_post_v5_asset_transfer_universal_transfer = privatePostV5AssetTransferUniversalTransfer = Entry('v5/asset/transfer/universal-transfer', 'private', 'POST', {'cost': 10})
    private_post_v5_asset_deposit_deposit_to_account = privatePostV5AssetDepositDepositToAccount = Entry('v5/asset/deposit/deposit-to-account', 'private', 'POST', {'cost': 5})
    private_post_v5_asset_withdraw_create = privatePostV5AssetWithdrawCreate = Entry('v5/asset/withdraw/create', 'private', 'POST', {'cost': 50})
    private_post_v5_asset_withdraw_cancel = privatePostV5AssetWithdrawCancel = Entry('v5/asset/withdraw/cancel', 'private', 'POST', {'cost': 50})
    private_post_v5_user_create_sub_member = privatePostV5UserCreateSubMember = Entry('v5/user/create-sub-member', 'private', 'POST', {'cost': 10})
    private_post_v5_user_create_sub_api = privatePostV5UserCreateSubApi = Entry('v5/user/create-sub-api', 'private', 'POST', {'cost': 10})
    private_post_v5_user_frozen_sub_member = privatePostV5UserFrozenSubMember = Entry('v5/user/frozen-sub-member', 'private', 'POST', {'cost': 10})
    private_post_v5_user_update_api = privatePostV5UserUpdateApi = Entry('v5/user/update-api', 'private', 'POST', {'cost': 10})
    private_post_v5_user_update_sub_api = privatePostV5UserUpdateSubApi = Entry('v5/user/update-sub-api', 'private', 'POST', {'cost': 10})
    private_post_v5_user_delete_api = privatePostV5UserDeleteApi = Entry('v5/user/delete-api', 'private', 'POST', {'cost': 10})
    private_post_v5_user_delete_sub_api = privatePostV5UserDeleteSubApi = Entry('v5/user/delete-sub-api', 'private', 'POST', {'cost': 10})
    private_post_v5_spot_lever_token_purchase = privatePostV5SpotLeverTokenPurchase = Entry('v5/spot-lever-token/purchase', 'private', 'POST', {'cost': 2.5})
    private_post_v5_spot_lever_token_redeem = privatePostV5SpotLeverTokenRedeem = Entry('v5/spot-lever-token/redeem', 'private', 'POST', {'cost': 2.5})
    private_post_v5_spot_margin_trade_switch_mode = privatePostV5SpotMarginTradeSwitchMode = Entry('v5/spot-margin-trade/switch-mode', 'private', 'POST', {'cost': 5})
    private_post_v5_spot_margin_trade_set_leverage = privatePostV5SpotMarginTradeSetLeverage = Entry('v5/spot-margin-trade/set-leverage', 'private', 'POST', {'cost': 5})
    private_post_v5_spot_cross_margin_trade_loan = privatePostV5SpotCrossMarginTradeLoan = Entry('v5/spot-cross-margin-trade/loan', 'private', 'POST', {'cost': 2.5})
    private_post_v5_spot_cross_margin_trade_repay = privatePostV5SpotCrossMarginTradeRepay = Entry('v5/spot-cross-margin-trade/repay', 'private', 'POST', {'cost': 2.5})
    private_post_v5_spot_cross_margin_trade_switch = privatePostV5SpotCrossMarginTradeSwitch = Entry('v5/spot-cross-margin-trade/switch', 'private', 'POST', {'cost': 2.5})
    private_post_v5_ins_loan_association_uid = privatePostV5InsLoanAssociationUid = Entry('v5/ins-loan/association-uid', 'private', 'POST', {'cost': 5})
    private_post_v5_lending_purchase = privatePostV5LendingPurchase = Entry('v5/lending/purchase', 'private', 'POST', {'cost': 5})
    private_post_v5_lending_redeem = privatePostV5LendingRedeem = Entry('v5/lending/redeem', 'private', 'POST', {'cost': 5})
    private_post_v5_lending_redeem_cancel = privatePostV5LendingRedeemCancel = Entry('v5/lending/redeem-cancel', 'private', 'POST', {'cost': 5})
    private_post_v5_account_set_collateral_switch = privatePostV5AccountSetCollateralSwitch = Entry('v5/account/set-collateral-switch', 'private', 'POST', {'cost': 5})
    private_post_v5_account_set_collateral_switch_batch = privatePostV5AccountSetCollateralSwitchBatch = Entry('v5/account/set-collateral-switch-batch', 'private', 'POST', {'cost': 5})
    private_post_v5_account_demo_apply_money = privatePostV5AccountDemoApplyMoney = Entry('v5/account/demo-apply-money', 'private', 'POST', {'cost': 5})



from ccxt.base.types import Entry


class ImplicitAPI:
    public_get_ohlc_pair = publicGetOhlcPair = Entry('ohlc/{pair}/', 'public', 'GET', {'cost': 1})
    public_get_order_book_pair = publicGetOrderBookPair = Entry('order_book/{pair}/', 'public', 'GET', {'cost': 1})
    public_get_ticker = publicGetTicker = Entry('ticker/', 'public', 'GET', {'cost': 1})
    public_get_ticker_hour_pair = publicGetTickerHourPair = Entry('ticker_hour/{pair}/', 'public', 'GET', {'cost': 1})
    public_get_ticker_pair = publicGetTickerPair = Entry('ticker/{pair}/', 'public', 'GET', {'cost': 1})
    public_get_transactions_pair = publicGetTransactionsPair = Entry('transactions/{pair}/', 'public', 'GET', {'cost': 1})
    public_get_trading_pairs_info = publicGetTradingPairsInfo = Entry('trading-pairs-info/', 'public', 'GET', {'cost': 1})
    public_get_currencies = publicGetCurrencies = Entry('currencies/', 'public', 'GET', {'cost': 1})
    public_get_eur_usd = publicGetEurUsd = Entry('eur_usd/', 'public', 'GET', {'cost': 1})
    public_get_travel_rule_vasps = publicGetTravelRuleVasps = Entry('travel_rule/vasps/', 'public', 'GET', {'cost': 1})
    private_get_travel_rule_contacts = privateGetTravelRuleContacts = Entry('travel_rule/contacts/', 'private', 'GET', {'cost': 1})
    private_get_contacts_contact_uuid = privateGetContactsContactUuid = Entry('contacts/{contact_uuid}/', 'private', 'GET', {'cost': 1})
    private_get_earn_subscriptions = privateGetEarnSubscriptions = Entry('earn/subscriptions/', 'private', 'GET', {'cost': 1})
    private_get_earn_transactions = privateGetEarnTransactions = Entry('earn/transactions/', 'private', 'GET', {'cost': 1})
    private_post_account_balances = privatePostAccountBalances = Entry('account_balances/', 'private', 'POST', {'cost': 1})
    private_post_account_balances_currency = privatePostAccountBalancesCurrency = Entry('account_balances/{currency}/', 'private', 'POST', {'cost': 1})
    private_post_balance = privatePostBalance = Entry('balance/', 'private', 'POST', {'cost': 1})
    private_post_balance_pair = privatePostBalancePair = Entry('balance/{pair}/', 'private', 'POST', {'cost': 1})
    private_post_bch_withdrawal = privatePostBchWithdrawal = Entry('bch_withdrawal/', 'private', 'POST', {'cost': 1})
    private_post_bch_address = privatePostBchAddress = Entry('bch_address/', 'private', 'POST', {'cost': 1})
    private_post_user_transactions = privatePostUserTransactions = Entry('user_transactions/', 'private', 'POST', {'cost': 1})
    private_post_user_transactions_pair = privatePostUserTransactionsPair = Entry('user_transactions/{pair}/', 'private', 'POST', {'cost': 1})
    private_post_crypto_transactions = privatePostCryptoTransactions = Entry('crypto-transactions/', 'private', 'POST', {'cost': 1})
    private_post_open_orders_all = privatePostOpenOrdersAll = Entry('open_orders/all/', 'private', 'POST', {'cost': 1})
    private_post_open_orders_pair = privatePostOpenOrdersPair = Entry('open_orders/{pair}/', 'private', 'POST', {'cost': 1})
    private_post_order_status = privatePostOrderStatus = Entry('order_status/', 'private', 'POST', {'cost': 1})
    private_post_cancel_order = privatePostCancelOrder = Entry('cancel_order/', 'private', 'POST', {'cost': 1})
    private_post_cancel_all_orders = privatePostCancelAllOrders = Entry('cancel_all_orders/', 'private', 'POST', {'cost': 1})
    private_post_cancel_all_orders_pair = privatePostCancelAllOrdersPair = Entry('cancel_all_orders/{pair}/', 'private', 'POST', {'cost': 1})
    private_post_buy_pair = privatePostBuyPair = Entry('buy/{pair}/', 'private', 'POST', {'cost': 1})
    private_post_buy_market_pair = privatePostBuyMarketPair = Entry('buy/market/{pair}/', 'private', 'POST', {'cost': 1})
    private_post_buy_instant_pair = privatePostBuyInstantPair = Entry('buy/instant/{pair}/', 'private', 'POST', {'cost': 1})
    private_post_sell_pair = privatePostSellPair = Entry('sell/{pair}/', 'private', 'POST', {'cost': 1})
    private_post_sell_market_pair = privatePostSellMarketPair = Entry('sell/market/{pair}/', 'private', 'POST', {'cost': 1})
    private_post_sell_instant_pair = privatePostSellInstantPair = Entry('sell/instant/{pair}/', 'private', 'POST', {'cost': 1})
    private_post_transfer_to_main = privatePostTransferToMain = Entry('transfer-to-main/', 'private', 'POST', {'cost': 1})
    private_post_transfer_from_main = privatePostTransferFromMain = Entry('transfer-from-main/', 'private', 'POST', {'cost': 1})
    private_post_my_trading_pairs = privatePostMyTradingPairs = Entry('my_trading_pairs/', 'private', 'POST', {'cost': 1})
    private_post_fees_trading = privatePostFeesTrading = Entry('fees/trading/', 'private', 'POST', {'cost': 1})
    private_post_fees_trading_market_symbol = privatePostFeesTradingMarketSymbol = Entry('fees/trading/{market_symbol}', 'private', 'POST', {'cost': 1})
    private_post_fees_withdrawal = privatePostFeesWithdrawal = Entry('fees/withdrawal/', 'private', 'POST', {'cost': 1})
    private_post_fees_withdrawal_currency = privatePostFeesWithdrawalCurrency = Entry('fees/withdrawal/{currency}/', 'private', 'POST', {'cost': 1})
    private_post_withdrawal_requests = privatePostWithdrawalRequests = Entry('withdrawal-requests/', 'private', 'POST', {'cost': 1})
    private_post_withdrawal_open = privatePostWithdrawalOpen = Entry('withdrawal/open/', 'private', 'POST', {'cost': 1})
    private_post_withdrawal_status = privatePostWithdrawalStatus = Entry('withdrawal/status/', 'private', 'POST', {'cost': 1})
    private_post_withdrawal_cancel = privatePostWithdrawalCancel = Entry('withdrawal/cancel/', 'private', 'POST', {'cost': 1})
    private_post_liquidation_address_new = privatePostLiquidationAddressNew = Entry('liquidation_address/new/', 'private', 'POST', {'cost': 1})
    private_post_liquidation_address_info = privatePostLiquidationAddressInfo = Entry('liquidation_address/info/', 'private', 'POST', {'cost': 1})
    private_post_btc_unconfirmed = privatePostBtcUnconfirmed = Entry('btc_unconfirmed/', 'private', 'POST', {'cost': 1})
    private_post_websockets_token = privatePostWebsocketsToken = Entry('websockets_token/', 'private', 'POST', {'cost': 1})
    private_post_btc_withdrawal = privatePostBtcWithdrawal = Entry('btc_withdrawal/', 'private', 'POST', {'cost': 1})
    private_post_btc_address = privatePostBtcAddress = Entry('btc_address/', 'private', 'POST', {'cost': 1})
    private_post_ripple_withdrawal = privatePostRippleWithdrawal = Entry('ripple_withdrawal/', 'private', 'POST', {'cost': 1})
    private_post_ripple_address = privatePostRippleAddress = Entry('ripple_address/', 'private', 'POST', {'cost': 1})
    private_post_ltc_withdrawal = privatePostLtcWithdrawal = Entry('ltc_withdrawal/', 'private', 'POST', {'cost': 1})
    private_post_ltc_address = privatePostLtcAddress = Entry('ltc_address/', 'private', 'POST', {'cost': 1})
    private_post_eth_withdrawal = privatePostEthWithdrawal = Entry('eth_withdrawal/', 'private', 'POST', {'cost': 1})
    private_post_eth_address = privatePostEthAddress = Entry('eth_address/', 'private', 'POST', {'cost': 1})
    private_post_xrp_withdrawal = privatePostXrpWithdrawal = Entry('xrp_withdrawal/', 'private', 'POST', {'cost': 1})
    private_post_xrp_address = privatePostXrpAddress = Entry('xrp_address/', 'private', 'POST', {'cost': 1})
    private_post_xlm_withdrawal = privatePostXlmWithdrawal = Entry('xlm_withdrawal/', 'private', 'POST', {'cost': 1})
    private_post_xlm_address = privatePostXlmAddress = Entry('xlm_address/', 'private', 'POST', {'cost': 1})
    private_post_pax_withdrawal = privatePostPaxWithdrawal = Entry('pax_withdrawal/', 'private', 'POST', {'cost': 1})
    private_post_pax_address = privatePostPaxAddress = Entry('pax_address/', 'private', 'POST', {'cost': 1})
    private_post_link_withdrawal = privatePostLinkWithdrawal = Entry('link_withdrawal/', 'private', 'POST', {'cost': 1})
    private_post_link_address = privatePostLinkAddress = Entry('link_address/', 'private', 'POST', {'cost': 1})
    private_post_usdc_withdrawal = privatePostUsdcWithdrawal = Entry('usdc_withdrawal/', 'private', 'POST', {'cost': 1})
    private_post_usdc_address = privatePostUsdcAddress = Entry('usdc_address/', 'private', 'POST', {'cost': 1})
    private_post_omg_withdrawal = privatePostOmgWithdrawal = Entry('omg_withdrawal/', 'private', 'POST', {'cost': 1})
    private_post_omg_address = privatePostOmgAddress = Entry('omg_address/', 'private', 'POST', {'cost': 1})
    private_post_dai_withdrawal = privatePostDaiWithdrawal = Entry('dai_withdrawal/', 'private', 'POST', {'cost': 1})
    private_post_dai_address = privatePostDaiAddress = Entry('dai_address/', 'private', 'POST', {'cost': 1})
    private_post_knc_withdrawal = privatePostKncWithdrawal = Entry('knc_withdrawal/', 'private', 'POST', {'cost': 1})
    private_post_knc_address = privatePostKncAddress = Entry('knc_address/', 'private', 'POST', {'cost': 1})
    private_post_mkr_withdrawal = privatePostMkrWithdrawal = Entry('mkr_withdrawal/', 'private', 'POST', {'cost': 1})
    private_post_mkr_address = privatePostMkrAddress = Entry('mkr_address/', 'private', 'POST', {'cost': 1})
    private_post_zrx_withdrawal = privatePostZrxWithdrawal = Entry('zrx_withdrawal/', 'private', 'POST', {'cost': 1})
    private_post_zrx_address = privatePostZrxAddress = Entry('zrx_address/', 'private', 'POST', {'cost': 1})
    private_post_gusd_withdrawal = privatePostGusdWithdrawal = Entry('gusd_withdrawal/', 'private', 'POST', {'cost': 1})
    private_post_gusd_address = privatePostGusdAddress = Entry('gusd_address/', 'private', 'POST', {'cost': 1})
    private_post_aave_withdrawal = privatePostAaveWithdrawal = Entry('aave_withdrawal/', 'private', 'POST', {'cost': 1})
    private_post_aave_address = privatePostAaveAddress = Entry('aave_address/', 'private', 'POST', {'cost': 1})
    private_post_bat_withdrawal = privatePostBatWithdrawal = Entry('bat_withdrawal/', 'private', 'POST', {'cost': 1})
    private_post_bat_address = privatePostBatAddress = Entry('bat_address/', 'private', 'POST', {'cost': 1})
    private_post_uma_withdrawal = privatePostUmaWithdrawal = Entry('uma_withdrawal/', 'private', 'POST', {'cost': 1})
    private_post_uma_address = privatePostUmaAddress = Entry('uma_address/', 'private', 'POST', {'cost': 1})
    private_post_snx_withdrawal = privatePostSnxWithdrawal = Entry('snx_withdrawal/', 'private', 'POST', {'cost': 1})
    private_post_snx_address = privatePostSnxAddress = Entry('snx_address/', 'private', 'POST', {'cost': 1})
    private_post_uni_withdrawal = privatePostUniWithdrawal = Entry('uni_withdrawal/', 'private', 'POST', {'cost': 1})
    private_post_uni_address = privatePostUniAddress = Entry('uni_address/', 'private', 'POST', {'cost': 1})
    private_post_yfi_withdrawal = privatePostYfiWithdrawal = Entry('yfi_withdrawal/', 'private', 'POST', {'cost': 1})
    private_post_yfi_address = privatePostYfiAddress = Entry('yfi_address', 'private', 'POST', {'cost': 1})
    private_post_audio_withdrawal = privatePostAudioWithdrawal = Entry('audio_withdrawal/', 'private', 'POST', {'cost': 1})
    private_post_audio_address = privatePostAudioAddress = Entry('audio_address/', 'private', 'POST', {'cost': 1})
    private_post_crv_withdrawal = privatePostCrvWithdrawal = Entry('crv_withdrawal/', 'private', 'POST', {'cost': 1})
    private_post_crv_address = privatePostCrvAddress = Entry('crv_address/', 'private', 'POST', {'cost': 1})
    private_post_algo_withdrawal = privatePostAlgoWithdrawal = Entry('algo_withdrawal/', 'private', 'POST', {'cost': 1})
    private_post_algo_address = privatePostAlgoAddress = Entry('algo_address/', 'private', 'POST', {'cost': 1})
    private_post_comp_withdrawal = privatePostCompWithdrawal = Entry('comp_withdrawal/', 'private', 'POST', {'cost': 1})
    private_post_comp_address = privatePostCompAddress = Entry('comp_address/', 'private', 'POST', {'cost': 1})
    private_post_grt_withdrawal = privatePostGrtWithdrawal = Entry('grt_withdrawal', 'private', 'POST', {'cost': 1})
    private_post_grt_address = privatePostGrtAddress = Entry('grt_address/', 'private', 'POST', {'cost': 1})
    private_post_usdt_withdrawal = privatePostUsdtWithdrawal = Entry('usdt_withdrawal/', 'private', 'POST', {'cost': 1})
    private_post_usdt_address = privatePostUsdtAddress = Entry('usdt_address/', 'private', 'POST', {'cost': 1})
    private_post_eurt_withdrawal = privatePostEurtWithdrawal = Entry('eurt_withdrawal/', 'private', 'POST', {'cost': 1})
    private_post_eurt_address = privatePostEurtAddress = Entry('eurt_address/', 'private', 'POST', {'cost': 1})
    private_post_matic_withdrawal = privatePostMaticWithdrawal = Entry('matic_withdrawal/', 'private', 'POST', {'cost': 1})
    private_post_matic_address = privatePostMaticAddress = Entry('matic_address/', 'private', 'POST', {'cost': 1})
    private_post_sushi_withdrawal = privatePostSushiWithdrawal = Entry('sushi_withdrawal/', 'private', 'POST', {'cost': 1})
    private_post_sushi_address = privatePostSushiAddress = Entry('sushi_address/', 'private', 'POST', {'cost': 1})
    private_post_chz_withdrawal = privatePostChzWithdrawal = Entry('chz_withdrawal/', 'private', 'POST', {'cost': 1})
    private_post_chz_address = privatePostChzAddress = Entry('chz_address/', 'private', 'POST', {'cost': 1})
    private_post_enj_withdrawal = privatePostEnjWithdrawal = Entry('enj_withdrawal/', 'private', 'POST', {'cost': 1})
    private_post_enj_address = privatePostEnjAddress = Entry('enj_address/', 'private', 'POST', {'cost': 1})
    private_post_alpha_withdrawal = privatePostAlphaWithdrawal = Entry('alpha_withdrawal/', 'private', 'POST', {'cost': 1})
    private_post_alpha_address = privatePostAlphaAddress = Entry('alpha_address/', 'private', 'POST', {'cost': 1})
    private_post_ftt_withdrawal = privatePostFttWithdrawal = Entry('ftt_withdrawal/', 'private', 'POST', {'cost': 1})
    private_post_ftt_address = privatePostFttAddress = Entry('ftt_address/', 'private', 'POST', {'cost': 1})
    private_post_storj_withdrawal = privatePostStorjWithdrawal = Entry('storj_withdrawal/', 'private', 'POST', {'cost': 1})
    private_post_storj_address = privatePostStorjAddress = Entry('storj_address/', 'private', 'POST', {'cost': 1})
    private_post_axs_withdrawal = privatePostAxsWithdrawal = Entry('axs_withdrawal/', 'private', 'POST', {'cost': 1})
    private_post_axs_address = privatePostAxsAddress = Entry('axs_address/', 'private', 'POST', {'cost': 1})
    private_post_sand_withdrawal = privatePostSandWithdrawal = Entry('sand_withdrawal/', 'private', 'POST', {'cost': 1})
    private_post_sand_address = privatePostSandAddress = Entry('sand_address/', 'private', 'POST', {'cost': 1})
    private_post_hbar_withdrawal = privatePostHbarWithdrawal = Entry('hbar_withdrawal/', 'private', 'POST', {'cost': 1})
    private_post_hbar_address = privatePostHbarAddress = Entry('hbar_address/', 'private', 'POST', {'cost': 1})
    private_post_rgt_withdrawal = privatePostRgtWithdrawal = Entry('rgt_withdrawal/', 'private', 'POST', {'cost': 1})
    private_post_rgt_address = privatePostRgtAddress = Entry('rgt_address/', 'private', 'POST', {'cost': 1})
    private_post_fet_withdrawal = privatePostFetWithdrawal = Entry('fet_withdrawal/', 'private', 'POST', {'cost': 1})
    private_post_fet_address = privatePostFetAddress = Entry('fet_address/', 'private', 'POST', {'cost': 1})
    private_post_skl_withdrawal = privatePostSklWithdrawal = Entry('skl_withdrawal/', 'private', 'POST', {'cost': 1})
    private_post_skl_address = privatePostSklAddress = Entry('skl_address/', 'private', 'POST', {'cost': 1})
    private_post_cel_withdrawal = privatePostCelWithdrawal = Entry('cel_withdrawal/', 'private', 'POST', {'cost': 1})
    private_post_cel_address = privatePostCelAddress = Entry('cel_address/', 'private', 'POST', {'cost': 1})
    private_post_sxp_withdrawal = privatePostSxpWithdrawal = Entry('sxp_withdrawal/', 'private', 'POST', {'cost': 1})
    private_post_sxp_address = privatePostSxpAddress = Entry('sxp_address/', 'private', 'POST', {'cost': 1})
    private_post_ada_withdrawal = privatePostAdaWithdrawal = Entry('ada_withdrawal/', 'private', 'POST', {'cost': 1})
    private_post_ada_address = privatePostAdaAddress = Entry('ada_address/', 'private', 'POST', {'cost': 1})
    private_post_slp_withdrawal = privatePostSlpWithdrawal = Entry('slp_withdrawal/', 'private', 'POST', {'cost': 1})
    private_post_slp_address = privatePostSlpAddress = Entry('slp_address/', 'private', 'POST', {'cost': 1})
    private_post_ftm_withdrawal = privatePostFtmWithdrawal = Entry('ftm_withdrawal/', 'private', 'POST', {'cost': 1})
    private_post_ftm_address = privatePostFtmAddress = Entry('ftm_address/', 'private', 'POST', {'cost': 1})
    private_post_perp_withdrawal = privatePostPerpWithdrawal = Entry('perp_withdrawal/', 'private', 'POST', {'cost': 1})
    private_post_perp_address = privatePostPerpAddress = Entry('perp_address/', 'private', 'POST', {'cost': 1})
    private_post_dydx_withdrawal = privatePostDydxWithdrawal = Entry('dydx_withdrawal/', 'private', 'POST', {'cost': 1})
    private_post_dydx_address = privatePostDydxAddress = Entry('dydx_address/', 'private', 'POST', {'cost': 1})
    private_post_gala_withdrawal = privatePostGalaWithdrawal = Entry('gala_withdrawal/', 'private', 'POST', {'cost': 1})
    private_post_gala_address = privatePostGalaAddress = Entry('gala_address/', 'private', 'POST', {'cost': 1})
    private_post_shib_withdrawal = privatePostShibWithdrawal = Entry('shib_withdrawal/', 'private', 'POST', {'cost': 1})
    private_post_shib_address = privatePostShibAddress = Entry('shib_address/', 'private', 'POST', {'cost': 1})
    private_post_amp_withdrawal = privatePostAmpWithdrawal = Entry('amp_withdrawal/', 'private', 'POST', {'cost': 1})
    private_post_amp_address = privatePostAmpAddress = Entry('amp_address/', 'private', 'POST', {'cost': 1})
    private_post_sgb_withdrawal = privatePostSgbWithdrawal = Entry('sgb_withdrawal/', 'private', 'POST', {'cost': 1})
    private_post_sgb_address = privatePostSgbAddress = Entry('sgb_address/', 'private', 'POST', {'cost': 1})
    private_post_avax_withdrawal = privatePostAvaxWithdrawal = Entry('avax_withdrawal/', 'private', 'POST', {'cost': 1})
    private_post_avax_address = privatePostAvaxAddress = Entry('avax_address/', 'private', 'POST', {'cost': 1})
    private_post_wbtc_withdrawal = privatePostWbtcWithdrawal = Entry('wbtc_withdrawal/', 'private', 'POST', {'cost': 1})
    private_post_wbtc_address = privatePostWbtcAddress = Entry('wbtc_address/', 'private', 'POST', {'cost': 1})
    private_post_ctsi_withdrawal = privatePostCtsiWithdrawal = Entry('ctsi_withdrawal/', 'private', 'POST', {'cost': 1})
    private_post_ctsi_address = privatePostCtsiAddress = Entry('ctsi_address/', 'private', 'POST', {'cost': 1})
    private_post_cvx_withdrawal = privatePostCvxWithdrawal = Entry('cvx_withdrawal/', 'private', 'POST', {'cost': 1})
    private_post_cvx_address = privatePostCvxAddress = Entry('cvx_address/', 'private', 'POST', {'cost': 1})
    private_post_imx_withdrawal = privatePostImxWithdrawal = Entry('imx_withdrawal/', 'private', 'POST', {'cost': 1})
    private_post_imx_address = privatePostImxAddress = Entry('imx_address/', 'private', 'POST', {'cost': 1})
    private_post_nexo_withdrawal = privatePostNexoWithdrawal = Entry('nexo_withdrawal/', 'private', 'POST', {'cost': 1})
    private_post_nexo_address = privatePostNexoAddress = Entry('nexo_address/', 'private', 'POST', {'cost': 1})
    private_post_ust_withdrawal = privatePostUstWithdrawal = Entry('ust_withdrawal/', 'private', 'POST', {'cost': 1})
    private_post_ust_address = privatePostUstAddress = Entry('ust_address/', 'private', 'POST', {'cost': 1})
    private_post_ant_withdrawal = privatePostAntWithdrawal = Entry('ant_withdrawal/', 'private', 'POST', {'cost': 1})
    private_post_ant_address = privatePostAntAddress = Entry('ant_address/', 'private', 'POST', {'cost': 1})
    private_post_gods_withdrawal = privatePostGodsWithdrawal = Entry('gods_withdrawal/', 'private', 'POST', {'cost': 1})
    private_post_gods_address = privatePostGodsAddress = Entry('gods_address/', 'private', 'POST', {'cost': 1})
    private_post_rad_withdrawal = privatePostRadWithdrawal = Entry('rad_withdrawal/', 'private', 'POST', {'cost': 1})
    private_post_rad_address = privatePostRadAddress = Entry('rad_address/', 'private', 'POST', {'cost': 1})
    private_post_band_withdrawal = privatePostBandWithdrawal = Entry('band_withdrawal/', 'private', 'POST', {'cost': 1})
    private_post_band_address = privatePostBandAddress = Entry('band_address/', 'private', 'POST', {'cost': 1})
    private_post_inj_withdrawal = privatePostInjWithdrawal = Entry('inj_withdrawal/', 'private', 'POST', {'cost': 1})
    private_post_inj_address = privatePostInjAddress = Entry('inj_address/', 'private', 'POST', {'cost': 1})
    private_post_rly_withdrawal = privatePostRlyWithdrawal = Entry('rly_withdrawal/', 'private', 'POST', {'cost': 1})
    private_post_rly_address = privatePostRlyAddress = Entry('rly_address/', 'private', 'POST', {'cost': 1})
    private_post_rndr_withdrawal = privatePostRndrWithdrawal = Entry('rndr_withdrawal/', 'private', 'POST', {'cost': 1})
    private_post_rndr_address = privatePostRndrAddress = Entry('rndr_address/', 'private', 'POST', {'cost': 1})
    private_post_vega_withdrawal = privatePostVegaWithdrawal = Entry('vega_withdrawal/', 'private', 'POST', {'cost': 1})
    private_post_vega_address = privatePostVegaAddress = Entry('vega_address/', 'private', 'POST', {'cost': 1})
    private_post_1inch_withdrawal = privatePost1inchWithdrawal = Entry('1inch_withdrawal/', 'private', 'POST', {'cost': 1})
    private_post_1inch_address = privatePost1inchAddress = Entry('1inch_address/', 'private', 'POST', {'cost': 1})
    private_post_ens_withdrawal = privatePostEnsWithdrawal = Entry('ens_withdrawal/', 'private', 'POST', {'cost': 1})
    private_post_ens_address = privatePostEnsAddress = Entry('ens_address/', 'private', 'POST', {'cost': 1})
    private_post_mana_withdrawal = privatePostManaWithdrawal = Entry('mana_withdrawal/', 'private', 'POST', {'cost': 1})
    private_post_mana_address = privatePostManaAddress = Entry('mana_address/', 'private', 'POST', {'cost': 1})
    private_post_lrc_withdrawal = privatePostLrcWithdrawal = Entry('lrc_withdrawal/', 'private', 'POST', {'cost': 1})
    private_post_lrc_address = privatePostLrcAddress = Entry('lrc_address/', 'private', 'POST', {'cost': 1})
    private_post_ape_withdrawal = privatePostApeWithdrawal = Entry('ape_withdrawal/', 'private', 'POST', {'cost': 1})
    private_post_ape_address = privatePostApeAddress = Entry('ape_address/', 'private', 'POST', {'cost': 1})
    private_post_mpl_withdrawal = privatePostMplWithdrawal = Entry('mpl_withdrawal/', 'private', 'POST', {'cost': 1})
    private_post_mpl_address = privatePostMplAddress = Entry('mpl_address/', 'private', 'POST', {'cost': 1})
    private_post_euroc_withdrawal = privatePostEurocWithdrawal = Entry('euroc_withdrawal/', 'private', 'POST', {'cost': 1})
    private_post_euroc_address = privatePostEurocAddress = Entry('euroc_address/', 'private', 'POST', {'cost': 1})
    private_post_sol_withdrawal = privatePostSolWithdrawal = Entry('sol_withdrawal/', 'private', 'POST', {'cost': 1})
    private_post_sol_address = privatePostSolAddress = Entry('sol_address/', 'private', 'POST', {'cost': 1})
    private_post_dot_withdrawal = privatePostDotWithdrawal = Entry('dot_withdrawal/', 'private', 'POST', {'cost': 1})
    private_post_dot_address = privatePostDotAddress = Entry('dot_address/', 'private', 'POST', {'cost': 1})
    private_post_near_withdrawal = privatePostNearWithdrawal = Entry('near_withdrawal/', 'private', 'POST', {'cost': 1})
    private_post_near_address = privatePostNearAddress = Entry('near_address/', 'private', 'POST', {'cost': 1})
    private_post_doge_withdrawal = privatePostDogeWithdrawal = Entry('doge_withdrawal/', 'private', 'POST', {'cost': 1})
    private_post_doge_address = privatePostDogeAddress = Entry('doge_address/', 'private', 'POST', {'cost': 1})
    private_post_flr_withdrawal = privatePostFlrWithdrawal = Entry('flr_withdrawal/', 'private', 'POST', {'cost': 1})
    private_post_flr_address = privatePostFlrAddress = Entry('flr_address/', 'private', 'POST', {'cost': 1})
    private_post_dgld_withdrawal = privatePostDgldWithdrawal = Entry('dgld_withdrawal/', 'private', 'POST', {'cost': 1})
    private_post_dgld_address = privatePostDgldAddress = Entry('dgld_address/', 'private', 'POST', {'cost': 1})
    private_post_ldo_withdrawal = privatePostLdoWithdrawal = Entry('ldo_withdrawal/', 'private', 'POST', {'cost': 1})
    private_post_ldo_address = privatePostLdoAddress = Entry('ldo_address/', 'private', 'POST', {'cost': 1})
    private_post_travel_rule_contacts = privatePostTravelRuleContacts = Entry('travel_rule/contacts/', 'private', 'POST', {'cost': 1})
    private_post_earn_subscribe = privatePostEarnSubscribe = Entry('earn/subscribe/', 'private', 'POST', {'cost': 1})
    private_post_earn_subscriptions_setting = privatePostEarnSubscriptionsSetting = Entry('earn/subscriptions/setting/', 'private', 'POST', {'cost': 1})
    private_post_earn_unsubscribe = privatePostEarnUnsubscribe = Entry('earn/unsubscribe', 'private', 'POST', {'cost': 1})
    private_post_wecan_withdrawal = privatePostWecanWithdrawal = Entry('wecan_withdrawal/', 'private', 'POST', {'cost': 1})
    private_post_wecan_address = privatePostWecanAddress = Entry('wecan_address/', 'private', 'POST', {'cost': 1})
    private_post_trac_withdrawal = privatePostTracWithdrawal = Entry('trac_withdrawal/', 'private', 'POST', {'cost': 1})
    private_post_trac_address = privatePostTracAddress = Entry('trac_address/', 'private', 'POST', {'cost': 1})
    private_post_eurcv_withdrawal = privatePostEurcvWithdrawal = Entry('eurcv_withdrawal/', 'private', 'POST', {'cost': 1})
    private_post_eurcv_address = privatePostEurcvAddress = Entry('eurcv_address/', 'private', 'POST', {'cost': 1})
    private_post_pyusd_withdrawal = privatePostPyusdWithdrawal = Entry('pyusd_withdrawal/', 'private', 'POST', {'cost': 1})
    private_post_pyusd_address = privatePostPyusdAddress = Entry('pyusd_address/', 'private', 'POST', {'cost': 1})
    private_post_lmwr_withdrawal = privatePostLmwrWithdrawal = Entry('lmwr_withdrawal/', 'private', 'POST', {'cost': 1})
    private_post_lmwr_address = privatePostLmwrAddress = Entry('lmwr_address/', 'private', 'POST', {'cost': 1})
    private_post_pepe_withdrawal = privatePostPepeWithdrawal = Entry('pepe_withdrawal/', 'private', 'POST', {'cost': 1})
    private_post_pepe_address = privatePostPepeAddress = Entry('pepe_address/', 'private', 'POST', {'cost': 1})
    private_post_blur_withdrawal = privatePostBlurWithdrawal = Entry('blur_withdrawal/', 'private', 'POST', {'cost': 1})
    private_post_blur_address = privatePostBlurAddress = Entry('blur_address/', 'private', 'POST', {'cost': 1})
    private_post_vext_withdrawal = privatePostVextWithdrawal = Entry('vext_withdrawal/', 'private', 'POST', {'cost': 1})
    private_post_vext_address = privatePostVextAddress = Entry('vext_address/', 'private', 'POST', {'cost': 1})
    private_post_cspr_withdrawal = privatePostCsprWithdrawal = Entry('cspr_withdrawal/', 'private', 'POST', {'cost': 1})
    private_post_cspr_address = privatePostCsprAddress = Entry('cspr_address/', 'private', 'POST', {'cost': 1})
    private_post_vchf_withdrawal = privatePostVchfWithdrawal = Entry('vchf_withdrawal/', 'private', 'POST', {'cost': 1})
    private_post_vchf_address = privatePostVchfAddress = Entry('vchf_address/', 'private', 'POST', {'cost': 1})
    private_post_veur_withdrawal = privatePostVeurWithdrawal = Entry('veur_withdrawal/', 'private', 'POST', {'cost': 1})
    private_post_veur_address = privatePostVeurAddress = Entry('veur_address/', 'private', 'POST', {'cost': 1})



from ccxt.base.types import Entry


class ImplicitAPI:
    public_get_getvalidprimarycurrencycodes = publicGetGetValidPrimaryCurrencyCodes = Entry('GetValidPrimaryCurrencyCodes', 'public', 'GET', {})
    public_get_getvalidsecondarycurrencycodes = publicGetGetValidSecondaryCurrencyCodes = Entry('GetValidSecondaryCurrencyCodes', 'public', 'GET', {})
    public_get_getvalidlimitordertypes = publicGetGetValidLimitOrderTypes = Entry('GetValidLimitOrderTypes', 'public', 'GET', {})
    public_get_getvalidmarketordertypes = publicGetGetValidMarketOrderTypes = Entry('GetValidMarketOrderTypes', 'public', 'GET', {})
    public_get_getvalidordertypes = publicGetGetValidOrderTypes = Entry('GetValidOrderTypes', 'public', 'GET', {})
    public_get_getvalidtransactiontypes = publicGetGetValidTransactionTypes = Entry('GetValidTransactionTypes', 'public', 'GET', {})
    public_get_getmarketsummary = publicGetGetMarketSummary = Entry('GetMarketSummary', 'public', 'GET', {})
    public_get_getorderbook = publicGetGetOrderBook = Entry('GetOrderBook', 'public', 'GET', {})
    public_get_getallorders = publicGetGetAllOrders = Entry('GetAllOrders', 'public', 'GET', {})
    public_get_gettradehistorysummary = publicGetGetTradeHistorySummary = Entry('GetTradeHistorySummary', 'public', 'GET', {})
    public_get_getrecenttrades = publicGetGetRecentTrades = Entry('GetRecentTrades', 'public', 'GET', {})
    public_get_getfxrates = publicGetGetFxRates = Entry('GetFxRates', 'public', 'GET', {})
    public_get_getorderminimumvolumes = publicGetGetOrderMinimumVolumes = Entry('GetOrderMinimumVolumes', 'public', 'GET', {})
    public_get_getcryptowithdrawalfees = publicGetGetCryptoWithdrawalFees = Entry('GetCryptoWithdrawalFees', 'public', 'GET', {})
    private_post_getopenorders = privatePostGetOpenOrders = Entry('GetOpenOrders', 'private', 'POST', {})
    private_post_getclosedorders = privatePostGetClosedOrders = Entry('GetClosedOrders', 'private', 'POST', {})
    private_post_getclosedfilledorders = privatePostGetClosedFilledOrders = Entry('GetClosedFilledOrders', 'private', 'POST', {})
    private_post_getorderdetails = privatePostGetOrderDetails = Entry('GetOrderDetails', 'private', 'POST', {})
    private_post_getaccounts = privatePostGetAccounts = Entry('GetAccounts', 'private', 'POST', {})
    private_post_gettransactions = privatePostGetTransactions = Entry('GetTransactions', 'private', 'POST', {})
    private_post_getfiatbankaccounts = privatePostGetFiatBankAccounts = Entry('GetFiatBankAccounts', 'private', 'POST', {})
    private_post_getdigitalcurrencydepositaddress = privatePostGetDigitalCurrencyDepositAddress = Entry('GetDigitalCurrencyDepositAddress', 'private', 'POST', {})
    private_post_getdigitalcurrencydepositaddresses = privatePostGetDigitalCurrencyDepositAddresses = Entry('GetDigitalCurrencyDepositAddresses', 'private', 'POST', {})
    private_post_gettrades = privatePostGetTrades = Entry('GetTrades', 'private', 'POST', {})
    private_post_getbrokeragefees = privatePostGetBrokerageFees = Entry('GetBrokerageFees', 'private', 'POST', {})
    private_post_getdigitalcurrencywithdrawal = privatePostGetDigitalCurrencyWithdrawal = Entry('GetDigitalCurrencyWithdrawal', 'private', 'POST', {})
    private_post_placelimitorder = privatePostPlaceLimitOrder = Entry('PlaceLimitOrder', 'private', 'POST', {})
    private_post_placemarketorder = privatePostPlaceMarketOrder = Entry('PlaceMarketOrder', 'private', 'POST', {})
    private_post_cancelorder = privatePostCancelOrder = Entry('CancelOrder', 'private', 'POST', {})
    private_post_synchdigitalcurrencydepositaddresswithblockchain = privatePostSynchDigitalCurrencyDepositAddressWithBlockchain = Entry('SynchDigitalCurrencyDepositAddressWithBlockchain', 'private', 'POST', {})
    private_post_requestfiatwithdrawal = privatePostRequestFiatWithdrawal = Entry('RequestFiatWithdrawal', 'private', 'POST', {})
    private_post_withdrawfiatcurrency = privatePostWithdrawFiatCurrency = Entry('WithdrawFiatCurrency', 'private', 'POST', {})
    private_post_withdrawdigitalcurrency = privatePostWithdrawDigitalCurrency = Entry('WithdrawDigitalCurrency', 'private', 'POST', {})



from ccxt.base.types import Entry


class ImplicitAPI:
    spot_kline_public_get_public_json = spotKlinePublicGetPublicJson = Entry('public.json', ['spot', 'kline', 'public'], 'GET', {'cost': 1})
    spot_kline_public_get_public_currency_json = spotKlinePublicGetPublicCurrencyJson = Entry('public{currency}.json', ['spot', 'kline', 'public'], 'GET', {'cost': 1})
    spot_v1_public_get_ping = spotV1PublicGetPing = Entry('ping', ['spot', 'v1', 'public'], 'GET', {'cost': 1})
    spot_v1_public_get_time = spotV1PublicGetTime = Entry('time', ['spot', 'v1', 'public'], 'GET', {'cost': 1})
    spot_v1_public_get_exchangeinfo = spotV1PublicGetExchangeInfo = Entry('exchangeInfo', ['spot', 'v1', 'public'], 'GET', {'cost': 1})
    spot_v1_public_get_depth = spotV1PublicGetDepth = Entry('depth', ['spot', 'v1', 'public'], 'GET', {'cost': 1, 'byLimit': [[100, 1], [500, 5], [1000, 10]]})
    spot_v1_public_get_trades = spotV1PublicGetTrades = Entry('trades', ['spot', 'v1', 'public'], 'GET', {'cost': 1})
    spot_v1_public_get_historicaltrades = spotV1PublicGetHistoricalTrades = Entry('historicalTrades', ['spot', 'v1', 'public'], 'GET', {'cost': 5})
    spot_v1_public_get_aggtrades = spotV1PublicGetAggTrades = Entry('aggTrades', ['spot', 'v1', 'public'], 'GET', {'cost': 1})
    spot_v1_public_get_ticker_24hr = spotV1PublicGetTicker24hr = Entry('ticker/24hr', ['spot', 'v1', 'public'], 'GET', {'cost': 1, 'noSymbol': 40})
    spot_v1_public_get_ticker_price = spotV1PublicGetTickerPrice = Entry('ticker/price', ['spot', 'v1', 'public'], 'GET', {'cost': 1, 'noSymbol': 2})
    spot_v1_public_get_ticker_bookticker = spotV1PublicGetTickerBookTicker = Entry('ticker/bookTicker', ['spot', 'v1', 'public'], 'GET', {'cost': 1, 'noSymbol': 2})
    spot_v1_public_get_market_kline = spotV1PublicGetMarketKline = Entry('market/kline', ['spot', 'v1', 'public'], 'GET', {'cost': 1})
    spot_v1_private_get_order = spotV1PrivateGetOrder = Entry('order', ['spot', 'v1', 'private'], 'GET', {'cost': 1})
    spot_v1_private_get_openorders = spotV1PrivateGetOpenOrders = Entry('openOrders', ['spot', 'v1', 'private'], 'GET', {'cost': 1})
    spot_v1_private_get_allorders = spotV1PrivateGetAllOrders = Entry('allOrders', ['spot', 'v1', 'private'], 'GET', {'cost': 5})
    spot_v1_private_get_account = spotV1PrivateGetAccount = Entry('account', ['spot', 'v1', 'private'], 'GET', {'cost': 5})
    spot_v1_private_get_mytrades = spotV1PrivateGetMyTrades = Entry('myTrades', ['spot', 'v1', 'private'], 'GET', {'cost': 5, 'noSymbol': 40})
    spot_v1_private_get_etf_net_value_symbol = spotV1PrivateGetEtfNetValueSymbol = Entry('etf/net-value/{symbol}', ['spot', 'v1', 'private'], 'GET', {'cost': 1})
    spot_v1_private_get_withdraw_history = spotV1PrivateGetWithdrawHistory = Entry('withdraw/history', ['spot', 'v1', 'private'], 'GET', {'cost': 1})
    spot_v1_private_get_deposit_history = spotV1PrivateGetDepositHistory = Entry('deposit/history', ['spot', 'v1', 'private'], 'GET', {'cost': 1})
    spot_v1_private_post_order = spotV1PrivatePostOrder = Entry('order', ['spot', 'v1', 'private'], 'POST', {'cost': 4})
    spot_v1_private_post_withdraw_commit = spotV1PrivatePostWithdrawCommit = Entry('withdraw/commit', ['spot', 'v1', 'private'], 'POST', {'cost': 1})
    spot_v1_private_delete_order = spotV1PrivateDeleteOrder = Entry('order', ['spot', 'v1', 'private'], 'DELETE', {'cost': 1})
    spot_v2_private_get_mytrades = spotV2PrivateGetMyTrades = Entry('myTrades', ['spot', 'v2', 'private'], 'GET', {'cost': 5})
    fapi_v1_public_get_ping = fapiV1PublicGetPing = Entry('ping', ['fapi', 'v1', 'public'], 'GET', {'cost': 1})
    fapi_v1_public_get_time = fapiV1PublicGetTime = Entry('time', ['fapi', 'v1', 'public'], 'GET', {'cost': 1})
    fapi_v1_public_get_contracts = fapiV1PublicGetContracts = Entry('contracts', ['fapi', 'v1', 'public'], 'GET', {'cost': 1})
    fapi_v1_public_get_depth = fapiV1PublicGetDepth = Entry('depth', ['fapi', 'v1', 'public'], 'GET', {'cost': 1})
    fapi_v1_public_get_ticker = fapiV1PublicGetTicker = Entry('ticker', ['fapi', 'v1', 'public'], 'GET', {'cost': 1})
    fapi_v1_public_get_klines = fapiV1PublicGetKlines = Entry('klines', ['fapi', 'v1', 'public'], 'GET', {'cost': 1})
    fapi_v2_private_get_mytrades = fapiV2PrivateGetMyTrades = Entry('myTrades', ['fapi', 'v2', 'private'], 'GET', {'cost': 1})
    fapi_v2_private_get_openorders = fapiV2PrivateGetOpenOrders = Entry('openOrders', ['fapi', 'v2', 'private'], 'GET', {'cost': 1})
    fapi_v2_private_get_order = fapiV2PrivateGetOrder = Entry('order', ['fapi', 'v2', 'private'], 'GET', {'cost': 1})
    fapi_v2_private_get_account = fapiV2PrivateGetAccount = Entry('account', ['fapi', 'v2', 'private'], 'GET', {'cost': 1})
    fapi_v2_private_get_leveragebracket = fapiV2PrivateGetLeverageBracket = Entry('leverageBracket', ['fapi', 'v2', 'private'], 'GET', {'cost': 1})
    fapi_v2_private_get_commissionrate = fapiV2PrivateGetCommissionRate = Entry('commissionRate', ['fapi', 'v2', 'private'], 'GET', {'cost': 1})
    fapi_v2_private_get_futures_transfer_history = fapiV2PrivateGetFuturesTransferHistory = Entry('futures_transfer_history', ['fapi', 'v2', 'private'], 'GET', {'cost': 1})
    fapi_v2_private_get_forceordershistory = fapiV2PrivateGetForceOrdersHistory = Entry('forceOrdersHistory', ['fapi', 'v2', 'private'], 'GET', {'cost': 1})
    fapi_v2_private_post_positionmargin = fapiV2PrivatePostPositionMargin = Entry('positionMargin', ['fapi', 'v2', 'private'], 'POST', {'cost': 1})
    fapi_v2_private_post_level_edit = fapiV2PrivatePostLevelEdit = Entry('level_edit', ['fapi', 'v2', 'private'], 'POST', {'cost': 1})
    fapi_v2_private_post_cancel = fapiV2PrivatePostCancel = Entry('cancel', ['fapi', 'v2', 'private'], 'POST', {'cost': 1})
    fapi_v2_private_post_order = fapiV2PrivatePostOrder = Entry('order', ['fapi', 'v2', 'private'], 'POST', {'cost': 1})
    fapi_v2_private_post_allopenorders = fapiV2PrivatePostAllOpenOrders = Entry('allOpenOrders', ['fapi', 'v2', 'private'], 'POST', {'cost': 1})
    fapi_v2_private_post_futures_transfer = fapiV2PrivatePostFuturesTransfer = Entry('futures_transfer', ['fapi', 'v2', 'private'], 'POST', {'cost': 1})
    dapi_v1_public_get_ping = dapiV1PublicGetPing = Entry('ping', ['dapi', 'v1', 'public'], 'GET', {'cost': 1})
    dapi_v1_public_get_time = dapiV1PublicGetTime = Entry('time', ['dapi', 'v1', 'public'], 'GET', {'cost': 1})
    dapi_v1_public_get_contracts = dapiV1PublicGetContracts = Entry('contracts', ['dapi', 'v1', 'public'], 'GET', {'cost': 1})
    dapi_v1_public_get_depth = dapiV1PublicGetDepth = Entry('depth', ['dapi', 'v1', 'public'], 'GET', {'cost': 1})
    dapi_v1_public_get_ticker = dapiV1PublicGetTicker = Entry('ticker', ['dapi', 'v1', 'public'], 'GET', {'cost': 1})
    dapi_v1_public_get_klines = dapiV1PublicGetKlines = Entry('klines', ['dapi', 'v1', 'public'], 'GET', {'cost': 1})
    dapi_v2_private_get_mytrades = dapiV2PrivateGetMyTrades = Entry('myTrades', ['dapi', 'v2', 'private'], 'GET', {'cost': 1})
    dapi_v2_private_get_openorders = dapiV2PrivateGetOpenOrders = Entry('openOrders', ['dapi', 'v2', 'private'], 'GET', {'cost': 1})
    dapi_v2_private_get_order = dapiV2PrivateGetOrder = Entry('order', ['dapi', 'v2', 'private'], 'GET', {'cost': 1})
    dapi_v2_private_get_account = dapiV2PrivateGetAccount = Entry('account', ['dapi', 'v2', 'private'], 'GET', {'cost': 1})
    dapi_v2_private_get_leveragebracket = dapiV2PrivateGetLeverageBracket = Entry('leverageBracket', ['dapi', 'v2', 'private'], 'GET', {'cost': 1})
    dapi_v2_private_get_commissionrate = dapiV2PrivateGetCommissionRate = Entry('commissionRate', ['dapi', 'v2', 'private'], 'GET', {'cost': 1})
    dapi_v2_private_get_futures_transfer_history = dapiV2PrivateGetFuturesTransferHistory = Entry('futures_transfer_history', ['dapi', 'v2', 'private'], 'GET', {'cost': 1})
    dapi_v2_private_get_forceordershistory = dapiV2PrivateGetForceOrdersHistory = Entry('forceOrdersHistory', ['dapi', 'v2', 'private'], 'GET', {'cost': 1})
    dapi_v2_private_post_positionmargin = dapiV2PrivatePostPositionMargin = Entry('positionMargin', ['dapi', 'v2', 'private'], 'POST', {'cost': 1})
    dapi_v2_private_post_level_edit = dapiV2PrivatePostLevelEdit = Entry('level_edit', ['dapi', 'v2', 'private'], 'POST', {'cost': 1})
    dapi_v2_private_post_cancel = dapiV2PrivatePostCancel = Entry('cancel', ['dapi', 'v2', 'private'], 'POST', {'cost': 1})
    dapi_v2_private_post_order = dapiV2PrivatePostOrder = Entry('order', ['dapi', 'v2', 'private'], 'POST', {'cost': 1})
    dapi_v2_private_post_allopenorders = dapiV2PrivatePostAllOpenOrders = Entry('allOpenOrders', ['dapi', 'v2', 'private'], 'POST', {'cost': 1})
    dapi_v2_private_post_futures_transfer = dapiV2PrivatePostFuturesTransfer = Entry('futures_transfer', ['dapi', 'v2', 'private'], 'POST', {'cost': 1})
    open_private_post_poseidon_api_v1_listenkey = openPrivatePostPoseidonApiV1ListenKey = Entry('poseidon/api/v1/listenKey', ['open', 'private'], 'POST', {'cost': 1})
    open_private_put_poseidon_api_v1_listenkey_listenkey = openPrivatePutPoseidonApiV1ListenKeyListenKey = Entry('poseidon/api/v1/listenKey/{listenKey}', ['open', 'private'], 'PUT', {'cost': 1})
    open_private_delete_poseidon_api_v1_listenkey_listenkey = openPrivateDeletePoseidonApiV1ListenKeyListenKey = Entry('poseidon/api/v1/listenKey/{listenKey}', ['open', 'private'], 'DELETE', {'cost': 1})
